<?php
require_once 'config.php';

//External networks by pvdc
$externalNets = array("Kingston-01"=>"vCloud-Internet-192","Oakville-01"=>"vCloud193");
//Database server
$dbhost = "localhost";
//Array of ServerCloud Canada PVDC's
$providerVdcs = array("Kingston-01","Oakville-01","StJohns-01","Moncton-01","Calgary-01","Vancouver-01");
$providerVDCShortNames = array("Kingston-01"=>"king1","Oakville-01"=>"oak1","StJohns-01"=>"stj1","Moncton-01"=>"mct1","Calgary-01"=>"cal1","Vancouver-01"=>"van1");

function createOrg($service,$orgName,$fullName) 
{
    //Create ORG in vCloud director
  
    $sdkAdmin = $service->createSDKAdminObj();

    // create an AdminOrgType data object
    $gSettings = new VMware_VCloud_API_OrgGeneralSettingsType();
    $gSettings->setCanPublishCatalogs("FALSE");

    $lSettings = new VMware_VCloud_API_OrgLdapSettingsType();
    $lSettings->setOrgLdapMode("NONE");

    $orgTemplateLease = new VMware_VCloud_API_OrgVAppTemplateLeaseSettingsType();
    $orgTemplateLease->setDeleteOnStorageLeaseExpiration(FALSE);
    $orgTemplateLease->setStorageLEaseSeconds(0);

    $orgLease = new VMware_VCloud_API_OrgLeaseSettingsType();
    $orgLease->setDeploymentLeaseSeconds(0);
    $orgLease->setStorageLeaseSeconds(0);
    $orgLease->setDeleteOnStorageLeaseExpiration(FALSE);

    $settings = new VMware_VCloud_API_OrgSettingsType();
    $settings->setOrgGeneralSettings($gSettings);
    $settings->setOrgLdapSettings($lSettings);
    $settings->setVAppLeaseSettings($orgLease);
    $settings->setVAppTemplateLeaseSettings($orgTemplateLease);

    $adminOrgObj = new VMware_VCloud_API_AdminOrgType();
    $adminOrgObj->set_name($orgName); // name is required
    $adminOrgObj->setFullName($fullName);  // FullName is required
    $adminOrgObj->setSettings($settings); // Settings is required
    $adminOrgObj->setIsEnabled("TRUE");

    //echo $adminOrgObj->export() . "\n";

    // create an administrative organization in vCloud Director.
    echo "Creating Organization " . $orgName . "...\n";
    $orgTask = $sdkAdmin->createOrganization($adminOrgObj);
    
    //Wait for tasks relating to ORG creation to complete
    $tasks = $orgTask->getTasks();
    if (!is_null($tasks))
    {
        $task = $tasks->getTask();
        if (sizeof($task) > 0)
        {
            $task = $service->waitForTask($task[0]);
            if ($task->get_status() != 'success')
                exit("Failed to create ORG.\n");
        }
    }
    echo "Organization created\n";
    
    return $orgTask;
}
   
function createVdc($service,$pvdcName,$orgName,$cpu,$memory,$storage1Amount,$storage2Amount,$storage3Amount)
{
    global $providerVDCShortNames;
    //Add vDC to ORG with specified parameters
    
    $vdcName = sprintf("%s-%s",$orgName,$providerVDCShortNames[$pvdcName]); //Set VDC name Org-Location eg: Philtopia-Kingston
  
    // create an SDK Admin object
    $sdkAdmin = $service->createSDKAdminObj();

    // get references to administrative organization entities
    $adminOrgRefs = $sdkAdmin->getAdminOrgRefs($orgName);
    if (0 == count($adminOrgRefs))
    {
        exit("No organization with $orgName is found\n");
    }
    $adminOrgRef = $adminOrgRefs[0];
    // create SDK AdminOrg object
    $sdkAdminOrg = $service->createSDKObj($adminOrgRef);

    // get references to the provider vDC
    $providerVdcRefs = $sdkAdmin->getProviderVdcRefs($pvdcName);
    if (0 == count($providerVdcRefs))
    {
        exit("No provider vDC with $pvdcName is found\n");
    }
    $providerVdcRef = $providerVdcRefs[0];
    $providerVdc = $service->createSDKObj($providerVdcRef);
    
    //Collect Storage profiles
    $pvdcStorageProfRefs = $providerVdc->getProviderVdcStorageProfileRefs();
  
    echo "Looking for storage policies in Provider VDC " . $pvdcName . "...\n";
    foreach ($pvdcStorageProfRefs as $profileRef) {
    	if(strstr($profileRef->get_name(),"Tier1")) {
        	echo "Tier 1 policy found: " . $profileRef->get_name() . "\n";
          	if(!isset($storageTier1))
	          $storageTier1 = $profileRef;
        }
      	if(strstr($profileRef->get_name(),"Tier2")) {
        	echo "Tier 2 policy found: " . $profileRef->get_name() . "\n";
            if(!isset($storageTier2))
          		$storageTier2 = $profileRef;
        }
      	if(strstr($profileRef->get_name(),"Tier3")) {
        	echo "Tier 3 policy found: " . $profileRef->get_name() . "\n";
         	if(!isset($storageTier3))
            	$storageTier3 = $profileRef;
        }
          
    }
    if(!isset($storage1Amount) || !isset($storage2Amount) || !isset($storage3Amount)) {
        //If ProfileRef is set set all policies to it otherwise exit because CreateVDCParams needs a storage profile
        if(isset($profileRef)) {
            $storageTier1 = $profileRef;
            $storageTier2 = $profileRef;
            $storageTier3 = $profileRef;
        } else {
            echo "No storage profiles found on this ORG, check configuration. Exiting...\n";
            exit(1);
        }
    }
    // create cpu
    $vcpu = new VMware_VCloud_API_CapacityWithUsageType();
    $vcpu->setUnits('MHz');
    $vcpu->setAllocated($cpu * 1000);
    $vcpu->setLimit($cpu * 1000);

    // create memory
    $mem = new VMware_VCloud_API_CapacityWithUsageType();
    $mem->setUnits('MB');
    $mem->setAllocated($memory * 1024);
    $mem->setLimit($memory * 1024);

    // configure compute capacity
    $cc = new VMware_VCloud_API_ComputeCapacityType();
    $cc->setCpu($vcpu);
    $cc->setMemory($mem);

    //Create Tiered profiles as needed
    $VdcStorageProfiles = array();
    if(isset($storage1Amount)) {
      	if($storage1Amount > 0) {
            if(isset($storageTier1)) {
	        $VdcStorageProfile = new VMware_VCloud_API_VdcStorageProfileParamsType();
                $VdcStorageProfile->setEnabled(true);
                $VdcStorageProfile->setProviderVdcStorageProfile($storageTier1);
                $VdcStorageProfile->setLimit($storage1Amount * 1024);
                $VdcStorageProfile->setUnits('MB');
                $VdcStorageProfile->setDefault(false);
                array_push($VdcStorageProfiles,$VdcStorageProfile);
            } else {
                echo "No Tier 1 policy to apply requested storage\n";
            }
        }
    }  
    if(isset($storage2Amount)) {
      	if($storage2Amount > 0) {
      	    if(isset($storageTier2)) {
                $VdcStorageProfile = new VMware_VCloud_API_VdcStorageProfileParamsType();
                $VdcStorageProfile->setEnabled(true);
                $VdcStorageProfile->setProviderVdcStorageProfile($storageTier2);
                $VdcStorageProfile->setLimit($storage2Amount * 1024);
                $VdcStorageProfile->setUnits('MB');
                $VdcStorageProfile->setDefault(false);
                array_push($VdcStorageProfiles,$VdcStorageProfile);
            } else {
                echo "No Tier 2 policy to apply requested storage\n";
            }  
        }
    }  
    if(isset($storage3Amount)) {
      	if($storage3Amount > 0) {
            if(isset($storageTier3)) {
                $VdcStorageProfile = new VMware_VCloud_API_VdcStorageProfileParamsType();
                $VdcStorageProfile->setEnabled(true);
                $VdcStorageProfile->setProviderVdcStorageProfile($storageTier3);
                $VdcStorageProfile->setLimit($storage3Amount * 1024);
                $VdcStorageProfile->setUnits('MB');
                $VdcStorageProfile->setDefault(false);
                array_push($VdcStorageProfiles,$VdcStorageProfile);
            } else {
                echo "No Tier 3 policy to apply requested storage\n";
            }  
        }
    }
    $VdcStorageProfiles[0]->setDefault(true); //set a default storage policy 
  
    //Find CloudIsolated networkpool for vdc
  	echo "Looking for VXLAN network pool in " . $pvdcName . "...\n";
    $networkPoolRefs = $providerVdc->getVMWNetworkPoolRefs();
    foreach($networkPoolRefs as $networkPoolRef) {
    	if(strstr($networkPoolRef->get_name(),"VXLAN")) {
            echo "Network pool " . $networkPoolRef->get_name() . " found\n";
          	$networkPool = $networkPoolRef;
        }
    }  
  
    // create a VMware_VCloud_API_AdminVdcType data object
    $vdc = new VMware_VCloud_API_CreateVdcParamsType();
    $vdc->set_name($vdcName);
    $vdc->setAllocationModel('AllocationPool');
    $vdc->setComputeCapacity($cc);
    $vdc->setVdcStorageProfile($VdcStorageProfiles);
    $vdc->setNicQuota(0); //No NIC limit
    $vdc->setNetworkQuota(0); //Needs to be set even if no networks are available
    if(isset($networkPool)) {
  	$vdc->setNetworkPoolReference($networkPool);
    	$vdc->setNetworkQuota(1000); //change it if network exists
    }
    $vdc->setIsEnabled(true);
    $vdc->setIsThinProvision(true); //Can't forget to do it if it's hard coded in
    $vdc->setUsesFastProvisioning(true);
    $vdc->setResourceGuaranteedCpu(0); //0% res on cpu
    $vdc->setResourceGuaranteedMemory(0.2); //20% on memory which is the min value for vcd
    $vdc->setVCpuInMhz(1000); //1GHZ
    $vdc->setVmQuota(100); 
    $vdc->setProviderVdcReference($providerVdcRef);    
    //echo $vdc->export();
  
    // create a vDC in an organization
    echo "Creating org vDC " . $vdcName . "...\n";
    $adminOrgVdc = $sdkAdminOrg->createAdminOrgVdc($vdc);
    $tasks = $adminOrgVdc->getTasks();
    if (!is_null($tasks))
    {
        $task = $tasks->getTask();
        if (sizeof($task) > 0)
        {
            $task = $service->waitForTask($task[0]);
            if ($task->get_status() != 'success')
                exit("Failed to create org vdc.\n");
        }
    }
    echo "Successfully created org vDC.\n";

    return $adminOrgVdc;
}

function getOrgNetwork($service,$orgName,$vdcName)
{
    //Return NetworkRef to first network in the vdc
    
	$orgRefs = $service->getOrgRefs($orgName);
	if (!empty($orgRefs))
	{
    	$sdkOrg  = $service->createSDKObj($orgRefs[0]);
    	$vdcRefs = $sdkOrg->getVdcRefs($vdcName);
    	if(!empty($vdcRefs))
    	{
        	foreach($vdcRefs as $vdcRef) 
            {
                $sdkVdc = $service->createSDKObj($vdcRef);
                $netRefs = $sdkVdc->getAvailableNetworkRefs();
            	if(!empty($netRefs)) 
                {
                	foreach($netRefs as $netRef) 
                    {
                        return $netRef;   
                
                    }
                }
            }
        }
    }
}

function getOrgNetworkByName($service,$orgName,$vdcName,$netName)
{
    //Return NetworkRef that matches name in the vdc

    $orgRefs = $service->getOrgRefs($orgName);
    if (!empty($orgRefs))
    {
        $sdkOrg  = $service->createSDKObj($orgRefs[0]);
        $vdcRefs = $sdkOrg->getVdcRefs($vdcName);
        if(!empty($vdcRefs))
        {
            foreach($vdcRefs as $vdcRef)
            {
                $sdkVdc = $service->createSDKObj($vdcRef);
                $netRefs = $sdkVdc->getAvailableNetworkRefs();
                if(!empty($netRefs))
                {
                    foreach($netRefs as $netRef)
                    {
                        if($netName == $netRef->get_name())
                            return $netRef;
                    }
                }
            }
        }
    }
}

function getOrgEdgeGW($service,$orgName,$vdcName) 
{
  	//Will return first EDGE gateway in Org/Vdc

    $orgRefs = $service->getOrgRefs($orgName);
	if (!empty($orgRefs))
	{
        $sdkOrg  = $service->createSDKObj($orgRefs[0]); //If theres ever an issue where the wrong org shows up look at this line
    	$vdcRefs = $sdkOrg->getVdcRefs($vdcName);
    	if(!empty($vdcRefs))
    	{
        	foreach($vdcRefs as $vdcRef) 
            {
            	$sdkVdc = $service->createSDKObj($vdcRef);
                $edgeRefs = $sdkVdc->getEdgeGatewayRefs();
            	if(!empty($edgeRefs)) 
                {
                	foreach($edgeRefs as $edgeRef) 
                    {

                    	return $edgeRef;
                	}
            	}
        	}
    	}
	}
}

function getOrgEdgeGWbyName($service,$orgName,$edgeGW)
{
    //Will return first EDGE gateway in Org/Vdc

    $orgRefs = $service->getOrgRefs($orgName);
    if (!empty($orgRefs))
    {
        $sdkOrg  = $service->createSDKObj($orgRefs[0]); //If theres ever an issue where the wrong org shows up look at this line
        $vdcRefs = $sdkOrg->getVdcRefs();
        if(!empty($vdcRefs))
        {
            foreach($vdcRefs as $vdcRef)
            {
                $sdkVdc = $service->createSDKObj($vdcRef);
                $edgeRefs = $sdkVdc->getEdgeGatewayRefs();
                if(!empty($edgeRefs))
                {
                    foreach($edgeRefs as $edgeRef)
                    {
                        if($edgeGW == $edgeRef->get_name())
                            return $edgeRef;
                    }
                }
            }
        }
    }
}
function createGatewayServices($service,$orgName,$vdcName,$edgeGateway)
{
    //Create Servercloud Canada base network services in ord/vdc gateway
  
    $edgeExtIp = getEdgeExtIp($service,$edgeGateway);
    
	$ipRange   = new VMware_VCloud_API_IpRangeType();
  	$ipRange->setStartAddress("192.168." . getOrgSubnet($vdcName) . ".100");
  	$ipRange->setEndAddress("192.168." . getOrgSubnet($vdcName) . ".150");
    
	$dhcpPool = new VMware_VCloud_API_DhcpPoolServiceType();
    $dhcpPool->setLowIpAddress("192.168." . getOrgSubnet($vdcName) . ".100");
  	$dhcpPool->setHighIpAddress("192.168." . getOrgSubnet($vdcName) . ".150");
    $dhcpPool->setDefaultLeaseTime(3600);
    $dhcpPool->setMaxLeaseTime(7200);
  	$dhcpPool->setNetwork(getOrgNetworkByName($service,$orgName,$vdcName,$vdcName)); //A provisioned Org netowrk will match the vdcName
    $dhcpPool->setIsEnabled(true);
  
    $dhcpService = new VMware_VCloud_API_GatewayDhcpServiceType();
    $dhcpService->setIsEnabled(true);
    $dhcpService->addPool($dhcpPool);  
   
    $firewallProtocols = new VMware_VCloud_API_Protocols();
    $firewallProtocols->setAny(true);
  
    $firewallRule = new VMware_VCloud_API_FirewallRuleType();
    $firewallRule->setSourceIP("192.168." . getOrgSubnet($vdcName) . ".0/24");
    $firewallRule->setDestinationIp("external");
    $firewallRule->setProtocols($firewallProtocols);
  	$firewallRule->setIsEnabled(true);
    
    $firewallService = new VMware_VCloud_API_FirewallServiceType();
  	$firewallService->setIsEnabled(true);
    $firewallService->setDefaultAction("drop");
    $firewallService->setLogDefaultAction(false);
  	$firewallService->addFirewallRule($firewallRule);
    
    $edgeGatewayObj = $service->createSDKObj($edgeGateway->get_href());
    $edgeGateway = $edgeGatewayObj->getEdgeGateway(); //replace gateway ref with gateway obj
  
    $configuration = $edgeGateway->getConfiguration();  //get current config
   
    $interfacesObj = $configuration->getGatewayInterfaces();
  	$interfaceObj = $interfacesObj->getGatewayInterface(); //sometimes this API seems wasteful
    foreach ($interfaceObj as $interface)
    {
		if($interface->getInterfaceType() == "uplink")
          $network = $interface->getNetwork();
    }
    
    $snatRule = new VMware_VCloud_API_GatewayNatRuleType();
    $snatRule->setInterface($network);
  	$snatRule->setOriginalIp("192.168." . getOrgSubnet($vdcName) . ".0/24"); 
    $snatRule->setTranslatedIp($edgeExtIp);
  
    $natRule = new VMware_VCloud_API_NatRuleType();
    $natRule->setRuleType("SNAT");
  	$natRule->setGatewayNatRule($snatRule);
    $natRule->setIsEnabled(true);
  
    $natService = new VMware_VCloud_API_NatServiceType();
  	$natService->addNatRule($natRule);
    $natService->setIsEnabled(true);
  
 	$gatewayFeatures = new VMware_VCloud_API_GatewayFeaturesType();
    $gatewayFeatures->addNetworkService($natService);
    $gatewayFeatures->addNetworkService($firewallService);
    $gatewayFeatures->addNetworkService($dhcpService);   
    
    $configuration->setEdgeGatewayServiceConfiguration($gatewayFeatures); //replace services with updated ones
  
    $edgeGateway->setConfiguration($configuration);
    echo "Adding DHCP, SNAT and Firewall services to Gateway...\n";
    $task = $edgeGatewayObj->modify($edgeGateway);
    
    $task = $service->waitForTask($task);
    if ($task->get_status() != 'success'){
        exit('Failed to update an edge Gateway.');
    } else {
    	echo "Services added successfully\n";
    }
  
} 

function createOrgNetwork($service,$orgName,$vdcName,$edgeGateway)
{
    //Create org vdc network
  
    $ipRange  = new VMware_VCloud_API_IpRangeType();
  	$ipRange->setStartAddress("192.168." . getOrgSubnet($vdcName) . ".2");
  	$ipRange->setEndAddress("192.168." . getOrgSubnet($vdcName) . ".99");
  	
    $ipRanges = new VMware_VCloud_API_IpRangesType();
    $ipRanges->addIpRange($ipRange);
    
    $ipScope  = new  VMware_VCloud_API_IpScopeType();
  	$ipScope->setGateway("192.168." . getOrgSubnet($vdcName) . ".1");
    $ipScope->setDns2("8.8.8.8");
  	$ipScope->setDns1("192.168." . getOrgSubnet($vdcName) . ".1"); 
    $ipScope->setIpRanges($ipRanges);
    $ipScope->setNetmask("255.255.255.0");
    $ipScope->setIsInherited(false);
  	$ipScope->setIsEnabled(true);

    $ipScopes = new VMware_VCloud_API_IpScopesType();
    $ipScopes->addIpScope($ipScope);
    
    $networkConfiguration = new VMware_VCloud_API_NetworkConfigurationType();
    $networkConfiguration->setIpScopes($ipScopes);
    $networkConfiguration->setFenceMode("natRouted");
    
    $orgVdcNetwork = new VMware_VCloud_API_OrgVdcNetworkType();
 	$orgVdcNetwork->set_name($vdcName);
    $orgVdcNetwork->setEdgeGateway($edgeGateway);
    $orgVdcNetwork->setConfiguration($networkConfiguration);  	
    
    echo "Org network config:\n";
  	echo "Name   : " . $orgVdcNetwork->get_name() . "\n";
  	echo "Gateway: " . $ipScope->getGateway() . "\n";
    echo "DNS1   : " . $ipScope->getDns1() . "\n";
    echo "DNS2   : " . $ipScope->getDns2() . "\n";
  	echo "Netmask: " . $ipScope->getNetmask() . "\n";
  	echo "Range  : " . $ipRange->getStartAddress() . "-" . $ipRange->getEndAddress() . "\n";
    echo "Creating ORG Network...\n";
    // create sdk admin object
    $sdkAdminObj = $service->createSDKAdminObj();

    // create admin org object
    $adminOrgRefs = $sdkAdminObj->getAdminOrgRefs($orgName);
    if(empty($adminOrgRefs))
    {
        exit("No admin org with name $orgName is found.");
    } 
  
    $adminOrgRef = $adminOrgRefs[0];
    //echo "-=" . $adminOrgRef->get_name() . "=-\n";
    $adminOrgObj = $service->createSDKObj($adminOrgRef->get_href());

    // create admin vdc object
    $adminVdcRefs = $adminOrgObj->getAdminVdcRefs($vdcName);
    if(empty($adminVdcRefs))
    {
        exit("No admin vdc with name $vdcName is found.");
    }
    //echo "-" . $adminVdcRefs[0]->get_name() . "-\n";
  	$adminVdcObj = $service->createSDKObj($adminVdcRefs[0]->get_href());
    
    $thisOrgVdcNetwork = $adminVdcObj->addvdcNetwork($orgVdcNetwork);
    
    $tasks = $thisOrgVdcNetwork->getTasks();
    if (!is_null($tasks))
    {
        $task = $tasks->getTask();
        if (sizeof($task) > 0)
        {
            $task = $service->waitForTask($task[0]);
            if ($task->get_status() != 'success')
                exit("Failed to create Org network.\n");
        }
    }
    
  	echo "Org network " . $orgVdcNetwork->get_name() . " created.\n";
}

function createEdgeGW($service,$orgName,$vdcName,$extNetName)
{
    //Create gateway in Org VDC with external network link and sub allocate external IP to gateway
  
    // create sdk admin object
    $sdkAdminObj = $service->createSDKAdminObj();

    // create admin org object
    $adminOrgRefs = $sdkAdminObj->getAdminOrgRefs($orgName);
    if(empty($adminOrgRefs))
    {
        exit("No admin org with name $orgName is found.");
    } 
  
    $adminOrgRef = $adminOrgRefs[0];
    //echo "-=" . $adminOrgRef->get_name() . "=-\n";
    $adminOrgObj = $service->createSDKObj($adminOrgRef->get_href());

    // create admin vdc object
    $adminVdcRefs = $adminOrgObj->getAdminVdcRefs($vdcName);
    if(empty($adminVdcRefs))
    {
        exit("No admin vdc with name $vdcName is found.");
    }
   // echo "-" . $adminVdcRefs[0]->get_name() . "-\n";
    $adminVdcObj = $service->createSDKObj($adminVdcRefs[0]->get_href());
   	// create a reference to the external network
    $extensionObj = $service->createSDKExtensionObj();
    $extNetRefs   = $extensionObj->getVMWExternalNetworks($extNetName);
    if (0 == count($extNetRefs))
    {
        exit("No external network with $extNetName is found!\n");
    }
    $extNetRef     = $extNetRefs[0];
    $extNetObj     = $service->createSDKObj($extNetRef->get_href());
    $configuration = $extNetObj->getVMWExternalNetwork()->getConfiguration();
    if(empty($configuration))
    {
        exit("$extNetName doesn't have any configuration.");
    }
    $ipScopes = $configuration->getIpScopes();
   
    if(empty($ipScopes))
    {
        exit("$extNetName doesn't have any IP Scopes.");
    }
    $ipScope = $ipScopes->getIpScope();
    $gateway = $ipScope[0]->getGateway();
    $netmask = $ipScope[0]->getNetmask();
    $ipRanges = $ipScope[0]->getIpRanges();
    if(empty($ipRanges))
    {
        exit("$extNetName doesn't have any IP Ranges.");
    }
   // $ipRange = $ipRanges->getIpRange();
   // $startAddress = $ipRange[0]->getStartAddress();
   // $endAddress = $ipRange[0]->getEndAddress();
    $extNetRef = VMware_VCloud_SDK_Helper::createReferenceTypeObj($extNetRef->get_href());
    
    //Assemble Edge Params
    $params = new VMware_VCloud_API_GatewayType();
    $params->set_Name($vdcName . "-GW");

    $gatcon=new VMware_VCloud_API_GatewayConfigurationType();
    $gatcon->setGatewayBackingConfig("compact");
    $gatinter=new VMware_VCloud_API_GatewayInterfaceType();
    $gatinter->setDisplayName("gateway");

    $gatinter->setNetwork($extNetRef);
    $gatinter->setInterfaceType("uplink");
    $gatinter->setInRateLimit("10");
  	$gatinter->setOutRateLimit("10");
  	$gatinter->setApplyRateLimit(true);
    $subnetparttype=new VMware_VCloud_API_SubnetParticipationType();
    $subnetparttype->setGateway($gateway);
    $subnetparttype->setNetmask($netmask);
   
    $gatinter->addSubnetParticipation($subnetparttype);
    $gatinter->setUseForDefaultRoute(true);

    $gatinfaces=new VMware_VCloud_API_GatewayInterfacesType();
    $gatinfaces->addGatewayInterface($gatinter);
    $gatcon->setGatewayInterfaces($gatinfaces);
    $gatcon->setHaEnabled(false);
    $gatcon->setUseDefaultRouteForDnsRelay(true);

    // Firewall Service
    $gatewayFeatures = new VMware_VCloud_API_GatewayFeaturesType();
    $firewallService = new VMware_VCloud_API_FirewallServiceType();
    $firewallService->setIsEnabled(true);
    $firewallService->setDefaultAction("drop");
    $firewallService->setLogDefaultAction(false);
    $gatewayFeatures->addNetworkService($firewallService);
  
    // NAT Service
    $natService = new VMware_VCloud_API_NatServiceType();
    $natService->setIsEnabled(false);
    $gatewayFeatures->addNetworkService($natService);

    // DHCP Service
    $dhcpService = new VMware_VCloud_API_DhcpServiceType();
    $gatewayFeatures->addNetworkService($dhcpService);
  
    $gatcon->setEdgeGatewayServiceConfiguration($gatewayFeatures);
    $params->setConfiguration($gatcon);
  
    echo "Edge Gateway config:\n";
    echo "Name   : " . $params->get_Name() . "\n";
    echo "Network: " . $extNetRefs[0]->get_name() . "\n";
    echo "Gateway: " . $gateway . "\n";
    echo "Netmask: " . $netmask . "\n";
    
    $edgeGateway = $adminVdcObj->createEdgeGateways($params);
    $tasks = $edgeGateway->getTasks();
    if (!is_null($tasks))
    {
        $task = $tasks->getTask();
        if (sizeof($task) > 0)
        {
            $task = $service->waitForTask($task[0]);
            if ($task->get_status() != 'success'){
                exit('Failed to create an edge Gateway: ' . $edgeGatewayName);
            }
        }
    }
     
    $edgeExtIp = getEdgeExtIp($service,$edgeGateway);
    echo "IP     : " . $edgeExtIp . "\n";
    echo "Edge gateway created\n";
    echo "Configuring Edge services...\n";
    echo "Adding sub allocation\n";
    
    $subAllocationipRange = new VMware_VCloud_API_IpRangeType();  
    $subAllocationipRange->setStartAddress($edgeExtIp);  
    $subAllocationipRange->setEndAddress($edgeExtIp);  
    $subAllocationipRanges = new VMware_VCloud_API_IpRangesType();  
    $subAllocationipRanges->setIpRange(array($subAllocationipRange));  
       
    $subnetparttype->setIpaddress($edgeExtIp);
    $subnetparttype->setIpRanges($subAllocationipRanges);
  	   
    $gatinter->setSubnetParticipation(array($subnetparttype)); //add back into Gateway interfaces type
    $gatcon->setGatewayInterfaces($gatinfaces);
    $params->setConfiguration($gatcon);
     
    $edgeGatewayObj = $service->createSDKObj($edgeGateway->get_href());
    $task = $edgeGatewayObj->modify($params);
  
    $task = $service->waitForTask($task);
    if ($task->get_status() != 'success'){
        exit('Failed to update an edge Gateway.');
    } else {
    	echo "Subnet added successfully\n";
    }
                                     
    return $vdcName . "-GW";
}  

function getEdgeExtIp($service,$edgeGateway)
{
    //Return the external IP of the edge gateway

    //so I had thought since I was using the variable created by the return of the edge device
    //that I could use that to get the external ip but no, somehow the EdgeGatewayType returned
    //by edge creation is not the current one so here I pull the config again. Wasteful
  
    $edgeGatewayObj = $service->createSDKObj($edgeGateway->get_href());
    $edgeGateway = $edgeGatewayObj->getEdgeGateway();

	$configuration            = $edgeGateway->getConfiguration();
  	$gatewayInterfaces        = $configuration->getGatewayInterfaces();
  	$gatewayInterface         = $gatewayInterfaces->getGatewayInterface();
    
    
    foreach ($gatewayInterface as $interface)
    {
		if($interface->getInterfaceType() == "uplink")
        {
        	$subnetParticipationType =  $interface->getSubnetParticipation();  
            return $subnetParticipationType[0]->getIpAddress();
        }
          
    }
    
    exit('Failed to detect external IP address: getEdgeExtIp()');
}

function getEdgeAllExtIp($service,$edgeGateway)
{
    //Return All external IP's of the edge gateway

    //so I had thought since I was using the variable created by the return of the edge device
    //that I could use that to get the external ip but no, somehow the EdgeGatewayType returned
    //by edge creation is not the current one so here I pull the config again. Wasteful

    $edgeGatewayObj = $service->createSDKObj($edgeGateway->get_href());
    $edgeGateway = $edgeGatewayObj->getEdgeGateway();

    $configuration            = $edgeGateway->getConfiguration();
    $gatewayInterfaces        = $configuration->getGatewayInterfaces();
    $gatewayInterface         = $gatewayInterfaces->getGatewayInterface();


    foreach ($gatewayInterface as $interface)
    {
        if($interface->getInterfaceType() == "uplink")
        {
            $subnetParticipationType =  $interface->getSubnetParticipation();
            return $subnetParticipationType;
        }

    }

    exit('Failed to detect external IP address: getEdgeExtIp()');
}

function getOrgSubnet($vdcName) 
{
  	//will return the 192.168.X.0 for the local network
	global $providerVdcs;
  
   	foreach ($providerVdcs as $subnet => $pvdc) {
		if(strstr($vdcName,$pvdc))
          return $subnet;
   	}
    return 0;
}

/*
* Database functions below
*/

function verifyQueue($data) 
{
	if($data['name'] && $data['orgName'] && $data['cpu'] && $data['ram'] && $data['location']) {
    	if($data['storage1Amount'] || $data['storage2Amount'] || $data['storage3Amount']) {
      		return 0;
      	} else {
      		doLog("ERROR no storage data attached\n" . var_dump($data),$data['id']); 
      	}
    } else
    	doLog("ERROR no initial data attached\n" . var_dump($data),$data['id']);
  
    setError($data['id']); //flag as error
    exit(1); //Can't continue with bad data 
}

function setWorking($id) 
{
	global $mysqli;
  
    if(!$id)
      return; //dont allow accidental setting of everything Working
  
  	$sql = "UPDATE queue SET status='1' WHERE id='$id'";
    if(!$mysqli->query($sql))
      echo "failed to update row " . $id . " with working status\n";
}

function setFinished($id) 
{
	global $mysqli;
  
    if(!$id)
      return; //dont allow accidental setting of everything finished
  
  	$sql = "UPDATE queue SET status='2' WHERE id='$id'";
    if(!$mysqli->query($sql))
      echo "failed to update row " . $id . " with finished status\n";
}

function setError($id) 
{
	global $mysqli;
  
    if(!$id)
      return; //dont allow accidental setting of everything error
  
  	$sql = "UPDATE queue SET status='3' WHERE id='$id'";
    if(!$mysqli->query($sql))
      echo "failed to update row " . $id . " with error status\n";
}

function getQueue() 
{
    global $mysqli;
  
    //Get next queue item from database
	$sql = "SELECT * FROM queue WHERE status='0' ORDER by id DESC";
    $result = $mysqli->query($sql);
    if($result->num_rows > 0) {
    	$row = $result->fetch_assoc();
      	return $row; //Return first result
    }
    return -1;
}

function checkBusy() {
  	global $mysqli;
  
	//Don't execute more than a couple jobs at a time, check how many are running
  	$sql = "SELECT * from queue WHERE status='1'";
    $result = $mysqli->query($sql);
    if($result->num_rows > 2) {
      	doLog("Queue full, quitting",0);
    	exit(0); //Quit
    }
}

function getDatabase() {
    global $dbhost;
      
	$mysqli = new mysqli($dbhost, "scc_form", 'Bcs$zc0!', "scc_provision");
    if ($mysqli->connect_errno) {
        echo "Failed to connect to MySQL: (" . $mysqli->connect_errno . ") " . $mysqli->connect_error;
    }
	return $mysqli;
}

function doLog($msg,$id)
{
	global $mysqli;

    //Write to log
    $sql = "INSERT INTO log (message,queue_id) VALUES ('$msg','$id')";
    $mysqli->query($sql);
}

function getStorageProfile($sdkVdc,$tier)
{
    //Collect Storage profiles
    $pvdcStorageProfRefs = $sdkVdc->getVdcStorageProfileRefs();

    echo "Looking for storage policies in VDC ...\n";
    foreach ($pvdcStorageProfRefs as $profileRef) {
        if($tier == 1 && strstr($profileRef->get_name(),"Tier1")) {
            echo "Tier 1 policy found: " . $profileRef->get_name() . "\n";
            return $profileRef;
        }
        if($tier == 2 && strstr($profileRef->get_name(),"Tier2")) {
            echo "Tier 2 policy found: " . $profileRef->get_name() . "\n";
            return $profileRef;
        }
        if($tier == 3 && strstr($profileRef->get_name(),"Tier3")) {
            echo "Tier 3 policy found: " . $profileRef->get_name() . "\n";
            return $profileRef;
        }
    }
}
?>
