#!/usr/bin/php
<?php
require_once dirname(__FILE__) . '/config.php';
require_once "functions.inc";

$mysqli = getDatabase();
checkBusy();
$nextJob = getQueue();

if($nextJob == -1)
  exit(0); //No work

verifyQueue($nextJob); //Make sure parameters are present

$name           = $nextJob['name'];
$orgName        = $nextJob['orgName'];
$pvdcName       = $nextJob['location'];
$cpu            = $nextJob['cpu'];
$memory         = $nextJob['ram'];
$storage1Amount = $nextJob['storage1Amount'];
$storage2Amount = $nextJob['storage2Amount'];
$storage3Amount = $nextJob['storage3Amount'];

//Start Provisioning process

// Initialize vCD parameters
$httpConfig = array('ssl_verify_peer'=>false, 'ssl_verify_host'=>false);

// login to vCD
$service = VMware_VCloud_SDK_Service::getService();
$service->login($server, array('username'=>$user, 'password'=>$pswd), $httpConfig, $sdkVersion);

setWorking($nextJob['id']); // flag row
doLog("Starting job ID: " . $nextJob['id'],$nextJob['id']);
try {
	//Create Org
	try {
		createOrg($service,$orgName,$name);
	} catch (Exception $e) {
		doLog("Failed to create Org\n" . $e,$nextJob['id']);
  		setError($nextJob['id']);
  		exit(1);
	}
	//Create VDC
	try {
		$vdc = createVdc($service,$pvdcName,$orgName,$cpu,$memory,$storage1Amount,$storage2Amount,$storage3Amount);
	} catch (Exception $e) {
		doLog("Failed to create vDC\n" . $e,$nextJob['id']);
  		setError($nextJob['id']);
  		exit(1);
	}
  
	//Prepare additional params for Network set up
	$externalNet = $externalNets[$pvdcName];
	if(!$externalNet) {
		dolog("No external network for Provider DC: " . $pvdcName . " skipping Edge gateway set up",$nextJob['id']);
  		setFinished($nextJob['id']);
  		exit(0);
	}

	$vdcName = $vdc->get_name();

	//Provision Edge gateway, org network and services
	try {
		createEdgeGW($service,$orgName,$vdcName,$externalNet); //Create Edge gateway linked to external network
		$edgeGateway = getOrgEdgeGW($service,$orgName,$vdcName); //get updated Ref (for some reason the return value on the created one is actually stale)
		createOrgNetwork($service,$orgName,$vdcName,$edgeGateway); //Create Org VDC network
		$edgeGateway = getOrgEdgeGW($service,$orgName,$vdcName);
		createGatewayServices($service,$orgName,$vdcName,$edgeGateway); //Add services to gateway/Org VDC network
	} catch (Exception $e) {
		doLog("Failed to create Gateway\n" . $e,$nextJob['id']);
  		setError($nextJob['id']);
  		exit(1);
	}

	setFinished($nextJob['id']); //All done
	doLog("All tasks completed for Job ID: " . $nextJob['id'],$nextJob['id']);

	// log out
	$service->logout();
} catch (Exception $e) {
	doLog("Runtime Exception:\n" . $e,$nextJob['id']);
	setError($nextJob['id']);
	exit(1);
}
?>
