#!/usr/bin/php
<?php
require_once dirname(__FILE__) . '/config.php';
require_once "functions.inc";

$mysqli = getDatabase();
checkSSLVPNBusy();
$nextJob = getSSLVPNQueue();

if($nextJob == -1)
    exit(0); //No work

verifySSLVPNQueue($nextJob); //Make sure parameters are present

$edgeID         = sprintf("edge-%s",$nextJob['edgeID']); //edge id's are edge-#
$ip             = $nextJob['ip']; //External Gateway IP
$port           = $nextJob['port']; //External Port
$cidr           = $nextJob['CIDR']; //internal network in CIDR format
$userName       = $nextJob['userName']; //initial username
$password       = $nextJob['password']; //initial password

//Start Provisioning process

// Initialize vCD parameters
$httpConfig = array('ssl_verify_peer'=>false, 'ssl_verify_host'=>false);

// login to vCD
$service = VMware_VCloud_SDK_Service::getService();
$service->login($server, array('username'=>$user, 'password'=>$pswd), $httpConfig, $sdkVersion);

setSSLVPNWorking($nextJob['id']); // flag row
doLog("Starting SSLVPN job ID: " . $nextJob['id'],$nextJob['id']);
try {
    //Create SSL VPN gateway
    try {
        createSSLVPNPlus($edgeID,$ip,$port,$cidr,"","","",$userName,$password); //Deploy SSLVPN
    } catch (Exception $e) {
        doLog("Failed to SSLVPN Gateway\n" . $e,$nextJob['id']);
        setSSLVPNError($nextJob['id']);
        exit(1);
    }

    setSSLVPNFinished($nextJob['id']); //All done
    doLog("All tasks completed for SSLVPN Job ID: " . $nextJob['id'],$nextJob['id']);

    // log out
    $service->logout();
} catch (Exception $e) {
    doLog("Runtime Exception:\n" . $e,$nextJob['id']);
    setSSLVPNError($nextJob['id']);
    exit(1);
}


function verifySSLVPNQueue($data)
{
    if($data['edgeID'] && $data['port'] && $data['ip'] && $data['CIDR'] && $data['userName'] && $data['password']) {
            return 0;
    } else
        doLog("SSLVPN ERROR no initial data attached\n" . var_dump($data),$data['id']);
    setError($data['id']); //flag as error
    exit(1); //Can't continue with bad data
}

function setSSLVPNWorking($id)
{
    global $mysqli;

    if(!$id)
        return; //dont allow accidental setting of everything Working

    $sql = "UPDATE sslvpnQueue SET status='1' WHERE id='$id'";
    if(!$mysqli->query($sql))
        echo "failed to update row " . $id . " with working status\n";
}

function setSSLVPNFinished($id)
{
    global $mysqli;

    if(!$id)
        return; //dont allow accidental setting of everything finished

    $sql = "UPDATE sslvpnQueue SET status='2' WHERE id='$id'";
    if(!$mysqli->query($sql))
        echo "failed to update row " . $id . " with finished status\n";
}

function setSSLVPNError($id)
{
    global $mysqli;

    if(!$id)
        return; //dont allow accidental setting of everything error

    $sql = "UPDATE sslvpnQueue SET status='3' WHERE id='$id'";
    if(!$mysqli->query($sql))
        echo "failed to update row " . $id . " with error status\n";
}

function getSSLVPNQueue()
{
    global $mysqli;

    //Get next queue item from database
    $sql = "SELECT * FROM sslvpnQueue WHERE status='0' ORDER by id DESC";
    $result = $mysqli->query($sql);
    if($result->num_rows > 0) {
        $row = $result->fetch_assoc();
        return $row; //Return first result
    }
    return -1;
}

function checkSSLVPNBusy() {
    global $mysqli;

    //Don't execute more than a couple jobs at a time, check how many are running
    $sql = "SELECT * from sslvpnQueue WHERE status='1'";
    $result = $mysqli->query($sql);
    if($result->num_rows > 2) {
        doLog("SSLVPN Queue full, quitting",0);
        exit(0); //Quit
    }
}

function createSSLVPNPlus($edgeId,$externalIp,$externalPort,$internalNetwork,$dns1,$dns2,$dnsSuffix,$username,$password)
{
    /*    $edgeId = "edge-114";
        $externalIp = "199.48.102.94";
        $externalPort = "443";
        $internalNetwork = "192.168.1.0/24";
        $dns1 = "192.168.2.1";
        $dns2 = "8.8.8.8";
        $dnsSuffix = "lab.local";
        $username = "vpnuser";
        $password = 'Bcs$zc0!';
    */
    $sslvpnconfig = <<<EOT
<sslvpnConfig><enabled>true</enabled><logging><enable>false</enable><logLevel>info</logLevel></logging><serverSettings>
<ip>$externalIp</ip><port>$externalPort</port><cipherList><cipher>AES128-SHA</cipher></cipherList><sslVersionList/></serverSettings>
<advancedConfig><enableCompression>false</enableCompression><forceVirtualKeyboard>false</forceVirtualKeyboard>
<randomizeVirtualkeys>false</randomizeVirtualkeys><preventMultipleLogon>false</preventMultipleLogon><clientNotification>
</clientNotification><enablePublicUrlAccess>false</enablePublicUrlAccess><timeout><forcedTimeout>0</forcedTimeout>
<sessionIdleTimeout>10</sessionIdleTimeout></timeout></advancedConfig><clientConfiguration><autoReconnect>true</autoReconnect>
<upgradeNotification>false</upgradeNotification></clientConfiguration><layoutConfiguration><portalTitle>VMware</portalTitle>
<companyName>VMware</companyName><logoExtention>jpg</logoExtention>
<logoUri>/3.0/edges/edge-1/sslvpn/config/layout/images/portallogo</logoUri><logoBackgroundColor>FFFFFF</logoBackgroundColor>
<titleColor>996600</titleColor><topFrameColor>000000</topFrameColor><menuBarColor>999999</menuBarColor>
<rowAlternativeColor>FFFFFF</rowAlternativeColor><bodyColor>FFFFFF</bodyColor><rowColor>F5F5F5</rowColor></layoutConfiguration>
<ipAddressPools><ipAddressPool><ipRange>192.168.10.100-192.168.10.150</ipRange><netmask>255.255.255.0</netmask>
<gateway>192.168.10.1</gateway><primaryDns>$dns1</primaryDns><secondaryDns>$dns2</secondaryDns>
<dnsSuffix>$dnsSuffix</dnsSuffix><winsServer></winsServer><description>VPN Pool</description><enabled>true</enabled>
</ipAddressPool></ipAddressPools><privateNetworks><privateNetwork><network>$internalNetwork</network><sendOverTunnel>
<optimize>true</optimize></sendOverTunnel><description>Internal network</description><enabled>true</enabled></privateNetwork>
</privateNetworks><users><user><userId>$username</userId><password>$password</password><firstName></firstName>
<lastName></lastName><description></description><disableUserAccount>false</disableUserAccount>
<passwordNeverExpires>false</passwordNeverExpires></user></users><clientInstallPackages><clientInstallPackage>
<profileName>install-package</profileName><gatewayList><gateway><hostName>$externalIp</hostName><port>$externalPort</port></gateway>
</gatewayList><createLinuxClient>true</createLinuxClient><createMacClient>true</createMacClient>
<startClientOnLogon>false</startClientOnLogon><hideSystrayIcon>false</hideSystrayIcon><rememberPassword>false</rememberPassword>
<silentModeOperation>false</silentModeOperation><silentModeInstallation>false</silentModeInstallation>
<hideNetworkAdaptor>false</hideNetworkAdaptor><createDesktopIcon>true</createDesktopIcon>
<enforceServerSecurityCertValidation>false</enforceServerSecurityCertValidation><description></description><enabled>true</enabled>
</clientInstallPackage></clientInstallPackages><authenticationConfiguration><passwordAuthentication>
<authenticationTimeout>1</authenticationTimeout><primaryAuthServers><localAuthServer><authServerType>local</authServerType>
<order>1</order><isSecondaryAuthServer>false</isSecondaryAuthServer><terminateSessionOnAuthFails>false</terminateSessionOnAuthFails>
<enabled>true</enabled><passwordPolicy><minLength>1</minLength><maxLength>63</maxLength><minAlphabets>0</minAlphabets>
<minDigits>0</minDigits><minSpecialChar>0</minSpecialChar><allowUserIdWithinPassword>true</allowUserIdWithinPassword>
<passwordLifeTime>365</passwordLifeTime><expiryNotification>25</expiryNotification></passwordPolicy></localAuthServer>
</primaryAuthServers><secondaryAuthServer/></passwordAuthentication></authenticationConfiguration></sslvpnConfig>
EOT;
    $url = "https://172.16.255.93/api/3.0/edges/$edgeId/sslvpn/config/";
    $testcommand = "/api/3.0/edges/edge-114/sslvpn/config/";
    $creds = "admin:5831bcs#";

    $headers = array(
        "Content-Type: application/xml",
        "Authorization: Basic " . base64_encode($creds)
    );

    $ch = curl_init();

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_HEADER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, false);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
    curl_setopt($ch, CURLOPT_POSTFIELDS, $sslvpnconfig);

    $data = curl_exec($ch);

    if (curl_errno($ch)) {
        curl_close($ch);
        return -1;
    } else {
        curl_close($ch);
        return 1;
    }
}
?>
