#!/usr/bin/php
<?php
require_once dirname(__FILE__) . '/config.php';
require_once 'functions.inc';

$shorts  = "";
$shorts .= "l:";
$shorts .= "o:";
$shorts .= "n:";
$shorts .= "c:";
$shorts .= "m:";
$shorts .= "1:";
$shorts .= "2:";
$shorts .= "3:";

$longs = array(
	"location:", //-l|--location
    "org:",      //-o|--org [required]
    "name:",      //-n|--name
    "cpu:",       //-c|--cpu
    "memory:",    //-m|--memory
    "tier1:",     //-1|--tier1
    "tier2:",     //-2|--tier2
    "tier3:",     //-3|--tier3
);

$pvdcName      = "Kingston-01";
$orgName       = null;
$fullName      = null;
$cpu           = "6.0";
$memory        = "8";
$storage1Amount = null;
$storage2Amount = null;
$storage3Amount = null;

$opts = getopt($shorts,$longs);

foreach (array_keys($opts) as $opt) switch ($opt)
{
	case "l":
    	$pvdcName = $opts['l'];
    	break;
  	case "location":
    	$pvdcName = $opts['location'];
    	break;
  	
  	case "o":
    	$orgName = $opts['o'];
    	break;
  	case "org":
    	$orgName = $opts["org"];
    	break;
    
  	case "n":
    	$fullName = $opts['n'];
    	break;
  	case "name":
    	$fullName = $opts['name'];
    	break;
  	
  	case "c":
    	$cpu = (int) $opts["c"];
    	break;
  	case "cpu":
    	$cpu = (int) $opts["cpu"];
    	break;
    
  	case "m":
    	$memory = (int) $opts["m"];
    	break;
  	case "memory":
    	$memory = (int) $opts["memory"];
    	break;
    
  	case "1":
    	$storage1Amount = $opts["1"];
    	break;
  	case "tier1":
        $storage1Amount = $opts["tier1"];
    	break;

  	case "2":
    	$storage2Amount = $opts["2"];
    	break;
  	case "tier2":
    	$storage2Amount = $opts["tier2"];
    	break;

  	case "3":
    	$storage3Amount = $opts["3"];
    	break;
  	case "tier3":
    	$storage3Amount = $opts["3"];
}

//Org required
if(!isset($orgName)) {
	echo "Error missing parameters\n";
    usage();
    exit(1);
}

//Set Full name or Org if not set
if (!isset($fullName)) $fullName = $orgName;

echo "Provision settings:\n";
echo "Org     : " . $orgName . "\n";
echo "Name    : " . $fullName . "\n";
echo "Location: " . $pvdcName . "\n";
echo "CPU     : " . $cpu . "\n";
echo "RAM     : " . $memory . "\n";
if(isset($storage1Amount)) echo "Tier1   : " . $storage1Amount . "\n";
if(isset($storage2Amount)) echo "Tier2   : " . $storage2Amount . "\n";
if(isset($storage3Amount)) echo "Tier3   : " . $storage3Amount . "\n";
sleep(5);

// Initialize parameters
$httpConfig = array('ssl_verify_peer'=>false, 'ssl_verify_host'=>false);

// login
$service = VMware_VCloud_SDK_Service::getService();
$service->login($server, array('username'=>$user, 'password'=>$pswd), $httpConfig, $sdkVersion);

//Create Org
createOrg($service,$orgName,$fullName);
//Create VDC
createVdc($service,$pvdcName,$orgName,$cpu,$memory,$storage1Amount,$storage2Amount,$storage3Amount);

// log out
$service->logout();
echo "logged out.\n";

function usage() 
{
	echo "provisionOrg.php usage:\n";
  	echo "-o|--org      Set organization name [REQUIRED]\n";
  	echo "-n|--name     Full name for organization. Use quotes if spaces present\n";
    echo "-l|--location ProviderVDC name to attach organization\n";
  	echo "-c|--cpu      CPU allocation in Ghz\n";
  	echo "-m|--memory   Memory allocation in GB\n";
    echo "-1|--tier1    Amount of Tier1 storage in GB\n";
  	echo "-2|--tier2    Amount of Tier2 storage in GB\n";
  	echo "-3|--tier3    Amount of Tier3 storage in GB\n\n";
  	echo "Example usage:\n";
  	echo "provisionOrg.php -o Example_Org -n \"Example Org\" -l Kingston-01 -c 12 -m 24 -1 50 -2 100 -3 500\n\n";
}

?>
