#!/usr/bin/php
<?php

$start = time();
require_once'Console/Table.php';
require_once 'config.php';

//establish database connection

$poweron      = 0;
$nocreate     = 0;
$suspended    = 0;
$waiting      = 0;
$unknown      = 0;
$unrecognized = 0;
$poweroff     = 0;
$inconsistent = 0;
$activeusers  = array();
$vApps        = 0;
$vms          = 0;
$orgs         = array();
$orgcache     = array(); //Cache the URI from vApp $record->get_org() with a previously resolved getFullName()

// login
$service = VMware_VCloud_SDK_Service::getService();
$service->login($server, array('username'=>$user, 'password'=>$pswd), $httpConfig,$sdkVersion);

// create an SDK Query object
$sdkQuery = VMware_VCloud_SDK_Query::getInstance($service);

//This Section will build the org info
//
$params = new VMware_VCloud_SDK_Query_Params();
$params->setPageSize(1000);
$recsObj = $sdkQuery->queryRecords("organization",$params);
foreach($recsObj->getRecord() as $orgRec) {
    $orgName = $orgRec->get_displayName();
    $orgs[$orgName]["VMs"]   = 0;
    $orgs[$orgName]["vApps"] = $orgRec->get_numberOfvApps();
    $orgs[$orgName]["Users"] = 0;
}

//This section will count vApps and build the activer user data
//
$params = new VMware_VCloud_SDK_Query_Params();
$params->setPageSize(128);
$page = 1;
while(1) {
    try {
        $recsObj = $sdkQuery->queryRecords("adminVApp",$params);
    } catch (Exception $e) {
        break;
    }
    foreach($recsObj->getRecord() as $record) {
        $vApps++;
        $orgURI = $record->get_org();
        if(!isset($orgcache[$orgURI])) {
            $sdkOrg   = $service->createSDKObj($orgURI);
            $orgRef   = $sdkOrg->getOrg();
            $orgcache[$orgURI] = $orgRef->getFullName();
        }
        //Count active users once
        $curUser = $record->get_OwnerName();
        if(!isset($activeusers[$curUser]) && $curUser != "system") {
            $activeusers[$curUser] = 1;
            $orgs[$orgcache[$orgURI]]["Users"]++;
        } elseif ($curUser != "system") {
            $activeusers[$curUser]++; //Log amount of vApps users have on the system except for system user
        }
        $orgs[$orgcache[$orgURI]]["VMs"] = $orgs[$orgcache[$orgURI]]["VMs"] + $record->get_numberOfVMs();
    }
    $page++;
    $params->setPage($page);
}

$page = 1;
$params->setPageSize(128);
$params->setPage($page);

while(1) {
    try {
        $recsObj = $sdkQuery->queryRecords("adminVM",$params);
    } catch (Exception $e) {
        break;
    }
    foreach($recsObj->getRecord() as $record) {
        $vms++;
        if(!$record->get_isVappTemplate()) {
            $status = $record->get_status();
            switch ($status) {
                case "FAILED_CREATION":
                    $nocreate++;
                    break;
                case "SUSPENDED":
                    $suspended++;
                    break;
                case "POWERED_ON":
                    $poweron++;
                    break;
                case "WAITING_FOR_INPUT":
                    $waiting++;
                    break;
                case "UNRESOLVED":
                    $unknown++;
                    break;
                case "UNRECOGNIZED_STATE":
                    $unrecognized++;
                    break;
                case "POWERED_OFF":
                    $poweroff++;
                    break;
                case "INCONSISTENT_STATE":
                    $inconsistent++;
                    break;
                default:
                    echo $status . "\n";
            }
        }
    }
    $page++;
    $params->setPage($page);
}

//vDC data
$page = 1;
$params->setPageSize(128);
$params->setPage($page);
while(1) {
    try {
        $recsObj = $sdkQuery->queryRecords("adminOrgVdc", $params);
    } catch (Exception $e) {
        break;
    }
    foreach ($recsObj->getRecord() as $record) {
        try {
            $orgName = $record->get_orgName();
            $orgRefs = $service->getOrgRefs($orgName);
            $ref = $orgRefs[0];
            $vDCArray[$record->get_name()] = array(round($record->get_storageUsedMB() / getAccurateStorageLimit($service, $ref) * 100), round($record->get_storageUsedMB() / 1000), round(getAccurateStorageLimit($service, $ref) / 1000), round($record->get_cpuUsedMhz() / 1000), round($record->get_cpuLimitMhz() / 1000), round($record->get_cpuUsedMhz() / $record->get_cpuLimitMhz() * 100), round($record->get_memoryUsedMB() / 1000), round($record->get_memoryLimitMB() / 1000), round($record->get_memoryUsedMB() / $record->get_memoryLimitMB() * 100));
        } catch (Exception $e) { }
    }
    $page++;
    $params->setPage($page);
}

/*
//Active Users
$activeReport = new Console_Table();
$activeReport->setHeaders(array("Top 5 Users","vApps"));
$activeReport->setAlign(1,CONSOLE_TABLE_ALIGN_CENTER);

arsort($activeusers);
$i = 5;
foreach($activeusers as $key => $value)
{
    $activeReport->addRow(array($key, $value));
    $i--;
    if($i <= 0)
        break;
}
$activeReport->addSeparator();
$activeReport->addRow(array("Total Active Users",count($activeusers)));
$activeReport->addRow(array("Total vApps",$vApps));
*/
//VM Report
$vmReport = new Console_Table();
$vmReport->setHeaders(array("VM State","Total VMs"));
$vmReport->addRow(array("Powered On",$poweron));
$vmReport->addRow(array("Powered Off",$poweroff));
$vmReport->addRow(array("Suspended",$suspended));
$vmReport->addRow(array("Cannot Create",$nocreate));
$vmReport->addRow(array("Waiting for input",$waiting));
$vmReport->addRow(array("Unknown State",$unknown));
$vmReport->addRow(array("Unrecognized State",$unrecognized));
$vmReport->addRow(array("Inconsistent State",$inconsistent));
$vmReport->setAlign(1,CONSOLE_TABLE_ALIGN_RIGHT);
$vmReport->calculateTotalsFor(array("1"));

//Organization Report
arsort($orgs);
$orgReport = new Console_Table();
$orgReport->setHeaders(array("Organization","vApps","VMs"));
$orgReport->setAlign(1,CONSOLE_TABLE_ALIGN_CENTER);
$orgReport->setAlign(2,CONSOLE_TABLE_ALIGN_CENTER);
foreach ($orgs as $orgKey => $values) {
    $orgReport->addRow(array($orgKey,$orgs[$orgKey]["vApps"],$orgs[$orgKey]["VMs"]));
}

//Datastore & vDC report
$vDCDatastoreReport = new Console_Table();
$vDCDatastoreReport->setHeaders(array("vDC", "Allocated (GB)", "Total (GB)", "%"));
$vDCReport = new Console_Table();
$vDCReport->setHeaders(array("vDC", "CPU usage", "CPU Total", "%", "RAM usage", "RAM Total", "%"));

arsort($vDCArray);
foreach ($vDCArray as $vDCKey => $value) {
    $vDCDatastoreReport->addRow(array($vDCKey, $value[1], $value[2], $value[0]));
    $vDCReport->addRow(array($vDCKey, $value[3], $value[4], $value[5], $value[6], $value[7], $value[8]));
}

//Datastore report
$datastoreReport = datastore_report("",$service);

//Main Report
$report = new Console_Table();
$report->setHeaders(array("vCloud Report 0.8"));
$report->setAlign(0,CONSOLE_TABLE_ALIGN_CENTER);
$report->addRow(array("Organization Report"));
$report->addRow(array($orgReport->getTable()));
$report->addRow(array("VM Report"));
$report->addRow(array($vmReport->getTable()));
$report->addRow(array("vDC Report"));
$report->addRow(array($vDCReport->getTable()));
$report->addRow(array("vDC Storage allocation"));
$report->addRow(array($vDCDatastoreReport->getTable()));
$report->addRow(array("Datastore Report"));
$report->addRow(array($datastoreReport->getTable()));
/*$report->addRow(array("Active User Report"));
$report->addRow(array($activeReport->getTable())); */
$end = time();
$report->addRow(array("Report generated in: " . round((($end - $start) / 60),2) . " minutes"));
echo $report->getTable();
echo "\n";
$service->logout();

function datastore_report($searchorg,$service)
{
    /*
    * I placed this in a seperate function for ease of reuse, in our vCloud, every ORG has the same datastores
    * so technically I only need to check one place but in more diverse cloud set ups there can be different
    * datastores per org so having this function take an ORG as parameter will make it flexible for future use
    * --Phil Spencer
    */
    global $dbkey;

    $datastoreReport = new Console_Table();
    $datastoreReport->setHeaders(array("Datastore","Used (GB)","Total (GB)","%"));
    $orgRef = $service->getOrgRefs($searchorg);
    if(empty($orgRef)) return -1;

    $SDKobj = $service->createSDKExtensionObj($orgRef[0]);
    $datastores = $SDKobj->getDatastores();
    if(empty($datastores)) return -1;
    foreach ($datastores as $datastore) {
        $datastoreArray[$datastore->getVcDisplayName()] = array(round($datastore->getUsedCapacityPercent()),round($datastore->getUsedCapacityGb()),round($datastore->getTotalCapacityGb()));
    }
    arsort($datastoreArray);
    foreach($datastoreArray as $datastoreKey => $value) {
        $datastoreReport->addRow(array($datastoreKey,$value[1],$value[2],$value[0]));
    }

    return $datastoreReport;
}

function getAccurateStorageLimit($service, $orgref)
{
    /* Work around for vCD 5.1 bug
     * this will check for enabled Storage profiles and only count thier limits
     */
    $accurateStorageLimit = 0;
    $sdkOrg = $service->createSDKObj($orgref);
    $vdcRefs = $sdkOrg->getVdcRefs();
    foreach ($vdcRefs as $vdcRef) {
        $sdkVdc = $service->createSDKObj($vdcRef);
        $vdcStorageProfiles = $sdkVdc->getVdcStorageProfiles();
        foreach ($vdcStorageProfiles as $vdcStorageProfile) {
            if ($vdcStorageProfile->getEnabled() == TRUE) {
                $accurateStorageLimit = $accurateStorageLimit + $vdcStorageProfile->getLimit();
            }
        }
    }
    return $accurateStorageLimit;

}
?>