#!/usr/bin/perl
# The Agent Master
# Phil Spencer 2013

use strict;
use Event;
use Socket;
use IO::Select;
use IO::Socket::INET;
use DBD::mysql;


my %OPTIONS = ( 
                DD_BUILD  => "0.0.4",
                DEBUG     => 1,
		);

my %DB_OPTIONS = (
                 DB_HOST => "localhost",
                 DB_PORT => 3306,
                 DB_USER => "reference",
                 DB_PASS => "T0Th3Cl0ud!",
                 DB_DB   => "agent_tracker"                 
                 );

my $SOCKET; #Main Socket
my $SELECT; 
my $MINUTE; #minute timer
my @CPOOL;  #Connection pool hashes
my $SQL;
my $REGION = 1;

main();

sub main()
{
  if (!$OPTIONS{DEBUG})
  {
     fork and exit;
  }
  do_log(sprintf('MAIN -> Agent Master Build %s', $OPTIONS{DD_BUILD}));
  server_init(); #Start the Master
  master_init(); #start timers and connect to SQL
  while (1) 
  {
    server_cycle();
    Event::sweep();
  }
}

sub master_init {
    my $time;
    my $data_source;
    my @present;
    my $sec;
    my $query;

    #Connect to database
    if(exists($DB_OPTIONS{DB_SOCK}))
    {
       $data_source = sprintf('DBI:mysql:database=%s;mysql_socket=%s', $DB_OPTIONS{DB_DB}, $DB_OPTIONS{DB_SOCK});
    }
    else
    {
      $data_source = sprintf('DBI:mysql:database=%s;host=%s;port=%d', $DB_OPTIONS{DB_DB}, $DB_OPTIONS{DB_HOST},$DB_OPTIONS{DB_PORT});
    }

    $SQL = DBI->connect($data_source, $DB_OPTIONS{DB_USER}, $DB_OPTIONS{DB_PASS});
    die if(!$SQL);	
    
    $time = time + (60 - (time % 60));
    $MINUTE = Event->timer(at=>$time, interval=>60,hard=>1,cb=>\&master_minute);

    $query = $SQL->prepare("DELETE FROM registered_agents"); #clear out registered agents as none should exist yet
    $query->execute();
}

sub master_minute {
  #Ping all connected clients
  my $pool;
  my $query;
  my $squery;
  my $row;

  $query = $SQL->prepare("SELECT * FROM registered_agents WHERE name='SSH' AND region=?");
  $query->execute($REGION); #keep SQL connection alive

  if($query->rows()) {
     #SSH connections detected
     while($row = $query->fetchrow_hashref()) {
        if ( (time - $$row{ping}) > 180 ) { #SSH detections timeout after X minutes 
            $squery = $SQL->prepare("DELETE FROM registered_agents WHERE name=? AND ip=?");
            $squery->execute($$row{name},$$row{ip});
            do_log(sprintf('MAIN -> Removing SSH agent %s',$$row{id}));
        }        
     }
  }

  foreach $pool (@CPOOL) {
     if ( (time - $$pool{ping}) > 180 ) {
	    #no response in 3 minutes
	    server_cleanup($$pool{handle});
     } else {
	    client_msg($pool,"PING");
     }	
  }
	
}

sub server_init() 
{
  $SELECT = new IO::Select;
  $SOCKET = new IO::Socket::INET( Proto     => "tcp",
                                  Listen    => 1000,
                                  LocalPort => "11000",
                                  Reuse     => "1"
                                );
  die "Could not create socket: $!\n" unless $SOCKET;
  $SELECT->add($SOCKET);
}

sub server_cycle()
{
   my $NEW_CONNECTION;
   my @ready;
   my $handle;
   my $data;
   my $client_address;

   @ready = $SELECT->can_read(.1);
  
   foreach $handle (@ready)
   {
      if ($handle == $SOCKET) #new connection time
      {
         $NEW_CONNECTION = $SOCKET->accept();
         do_log(sprintf("CONNECTION -> NEW from %s", $NEW_CONNECTION->peerhost()));
         $SELECT->add($NEW_CONNECTION);
      } else {
         if(sysread($handle, $data, 512) == 0) #read it or close it
         {
            #report error and cleanup any clients associated with that connection 
            if ($OPTIONS{DEBUG}) { do_log("ERROR -> READ failed on socket closing connection\n") }
            server_cleanup($handle);
         } else {
            server_recieve($handle,$data);
         }
  
      }
   }
}

sub server_recieve #\$handle,$data
{
   my $handle = $_[0];
   my $data = $_[1];
   my @message;
   my %source;
   my $message;
   my $pool;
   
   @message = split(/\s+/,$data);
   if ($message[0] eq "REGISTER") {
       if ($message[1]) { # && !is_registered($handle->peerhost())) {
          $source{handle} = $handle;
          $source{nickname} = $message[1];
          $source{ip} = $handle->peerhost();
          register_client(\%source);
       } else {
	   send($handle,"ERROR: Registration failed due to nickname error\n",0);
       }
          	
   }
   if ($message[0] eq "LOGOUT") {
      server_cleanup($handle);   	
   }
   if ($message[0] eq "PONG") {
      server_pong(getsource($handle));	
   }
}

sub server_pong {
   my $source = $_[0];
   my $query;

   $$source{ping} = time;
   $query = $SQL->prepare("UPDATE registered_agents SET ping=? WHERE name=? AND ip=?");
   $query->execute(time,$$source{nickname},$$source{ip});	
}

sub server_cleanup #\$handle
{
    #if a client disconnects compare its handle against clients and remove any
    #matching clients from the CPOOL
    my $handle = $_[0];
    my @DISCARD;
    my $discard;
    my $pool;

    foreach $pool (@CPOOL)
    {
       if($handle == $$pool{handle}) {
          #Originally I had unregister_client here but that changes the length of
          #@CPOOL and then clients are missed so I created a DISCARD array for everyone
          #I need to get rid of.
          push @DISCARD, $pool;
       }
    }
    foreach $discard (@DISCARD) {
       unregister_client($discard); #Good bye!
    }
    $SELECT->remove($handle);
    $handle->close;
}

sub client_msg # \%source, $msg
{
   my $source = $_[0];
   my $msg    = $_[1];
   my $nickname;
   my @lines;
   my $line;
   my $handle;
   
   @lines = split(/[\n\r]+/, $msg);
 
   return if (!$source);

   if($$source{handle})
   {
      foreach $line (@lines)
      {
         if ($OPTIONS{DEBUG}) { do_log(sprintf("SEND %s %s -> %s", $$source{nickname},$$source{ip}, $line)) }
         if (!send($$source{handle}, $line . "\n", 0))
         {
           if ($OPTIONS{DEBUG}) { do_log("ERROR -> SEND failed on socket closing connection\n") }
          server_cleanup($$source{handle});
         }
      }
   }
}   

sub global_msg #\$msg
{
   my $msg    = $_[0];
   my $handle;
   my @handles;
   
   @handles = $SELECT->handles;
   foreach $handle (@handles) {
    send($handle, "*** " . $msg . "\n", 0);
   }
} 

sub register_client #\%source
{
  #We just need to add $source to @cpool so when needed we can find this client later
  my $source = $_[0];
  my $query;
  
  $$source{ping} = time;
  push @CPOOL, $source;
  client_msg($source,"Logged In.");
  $query = $SQL->prepare("INSERT INTO registered_agents SET name=?,ip=?,ping=?,region=?");
  $query->execute($$source{nickname},$$source{ip},time,$REGION); 
  if($$source{nickname} ne "SSH") {#SSH isn't going to trigger an occupancy as it allows multiple connections
     register_occupied($source);
  }
}  

sub register_occupied
{
  #Flag registered vApp as occupied
  my $source = $_[0];
  my $query;   
  my $row;
  
  #Check if there is a vApp
  $query = $SQL->prepare("SELECT vapp_id FROM registered_vms WHERE ip=?");
  $query->execute($$source{ip});

  return if(!$query->rows()); #No VMs somehow, erroneous agent
  
  $row = $query->fetchrow_hashref();
  
  $$source{vapp_id} = $$row{vapp_id};

  $query = $SQL->prepare("UPDATE registered_vapps SET occupied='1' WHERE id=?");
  $query->execute($$source{vapp_id});
}

sub unregister_client #\%source
{
   #take leaving clients out of the hash pool
   #This does not disconnect the client as the client may be connected to a hub
   #client that accepts multiple connections

   my $source = $_[0];
   my $i;
   my $query;

   return if (!$source);

   for(my $i = 0; $i < scalar @CPOOL; $i++)
   {
      if($CPOOL[$i] == $source)
      {
         if ($OPTIONS{DEBUG}) { do_log(sprintf("MAIN -> %s %s logging off",$$source{nickname},$$source{ip})) }
         splice(@CPOOL, $i, 1);
         if($$source{nickname} ne "SSH") { #SSH will hang around for 30 min per connection, detection of occupancy is too annoying if it becomes super critical it can be revisted
            $query = $SQL->prepare("DELETE FROM registered_agents WHERE name=? AND ip=?");
            $query->execute($$source{nickname},$$source{ip});
            $query = $SQL->prepare("UPDATE registered_vapps SET occupied='0' WHERE id=?");
            $query->execute($$source{vapp_id});
         }
      }
   }
   return;
}

sub show_cpool
{
   #displays nicks in cpool so I can tell if it is deleting items properly
   my $pool;
   foreach $pool (@CPOOL)
   {
      printf("connection: %s\n",$$pool{nickname});
   }
   return;
}

sub getsource #\$id
{
   my $handle = $_[0];
   my $pool;

   foreach $pool (@CPOOL)
   {
      if($$pool{handle} == $handle)
      {
         return $pool;
      }
   }
   return 0;
}

sub is_registered #\$nickname
{
   my $nickname = $_[0];
   my $pool;

   foreach $pool (@CPOOL)
   {
      if($$pool{ip} eq $nickname)
      {
         return 1;
      }
   }
   return 0;
}

sub do_log #($data)
{
   my $data  = $_[0];
   my $query;

   print STDOUT "[" . scalar localtime() . "] " . $data . "\n";
   if($SQL) {
      $query = $SQL->prepare("INSERT INTO logs SET source=?,log=?");
      $query->execute("AGENT SERVER",$data);
   }
}

