var dmc = (function(){
    var devicePluginController = (function(){
        var normalizeArray = function(array) {
            // 'unknown' is not a valid JavaScript type, so if the condition is met we can safely assume
            // JScript (IE).
            return typeof(array) == 'unknown' ? array.toArray() : array;
        };

        var getVirtualDevice = function(type) {
            vmware.log("TRACE", "plugin", "Getting virtual devices of type {0}.".format(type));

            var devices = normalizeArray(deviceControl.getVirtualDevices(type));

            if (devices.length == 0) {
                return false;
            }

            return devices[0];
        };

        var getVirtualDeviceDetails = function (virtualDevice) {
            vmware.log("TRACE", "plugin", "Retrieving details for virtual device {0}.".format(virtualDevice));

            var details = false;
            try {
                details = deviceControl.getVirtualDeviceDetails(virtualDevice);
                vmware.log("TRACE", "plugin", "Details for virtual device {0}: {1}".format(virtualDevice, vmware.log.stringify(details)));
            } catch (error) {
                vmware.log("DEBUG", "plugin", "Error encontered while retrieving details for virtual device {0}: {1}".format(virtualDevice, error));
            }

            return details;
        };

        var isVirtualDeviceConnected = function (virtualDevice) {
            var details = getVirtualDeviceDetails(virtualDevice);
            if (details) {
                return details.state == deviceControl.DeviceState.DS_CONNECTED;
            }
            return false;
        };

        var isVirtualDeviceConnectedByMe = function(virtualDevice) {
            var details = getVirtualDeviceDetails(virtualDevice);
            if (details) {
                return details.connectedByMe;
            }
            return false;
        };

        var isPhysicalDeviceConnectedByMe = function(physicalDevice) {
            var details = getPhysicalDeviceDetails(physicalDevice);
            if (details) {
                return details.connectedByMe;
            }
            return false;
        };

        var isFileConnected = function(virtualDevice) {
            var details = getVirtualDeviceDetails(virtualDevice);
            if (details) {
                if (!details.connectedByMe) {
                    return false;
                }

                if (details.backing == deviceControl.DeviceBacking.DB_FILE) {
                    return true;
                }

                // we can't always believe VMRC when it reports a physical
                // device is connected (bug 885578), so we'll double check
                var physicalDevices = getPhysicalDevices(details.type);
                if (physicalDevices) {
                    for (var i = 0; i < physicalDevices.length; i++) {
                        if (details.backingKey == physicalDevices[i]) {
                            return false;
                        }
                    }
                    return true;
                }
            }
            return false;
        };

        var getFileConnected = function(virtualDevice) {
            if (!isVirtualDeviceConnectedByMe(virtualDevice) || !isFileConnected(virtualDevice)) {
                return "";
            }

            var details = getVirtualDeviceDetails(virtualDevice);
            if (details) {
                return details.backingKey;
            }
            return false;
        };

        var hasClientBacking = function(virtualDevice) {
            var details = getVirtualDeviceDetails(virtualDevice);
            if (details) {
                return details.clientBacking;
            }
            return true;
        };

        var getPhysicalDevices = function(deviceType) {
            vmware.log("TRACE", "plugin", "Attempting to getPhysicalClientDevices.");

            try {
                return normalizeArray(deviceControl.getPhysicalClientDevices(deviceType));
            } catch(error) {
                vmware.log("DEBUG", "plugin", "Error encontered when getting client devices: {0}".format(error));
                // This error was thrown either because Deny was clicked or because of an actual underlying error.
                // Either way the popout console should be restarted before device attachment that is NOT file-based can be performed.
                return false;
            }
        };

        var getPhysicalDeviceDetails = function (physicalDevice) {
            vmware.log("TRACE", "plugin", "Retrieving details for physical device {0}.".format(physicalDevice));

            var details = false;
            try {
                details = deviceControl.getPhysicalClientDeviceDetails(physicalDevice);
                vmware.log("TRACE", "plugin", "Details for virtual device {0}: {1}".format(physicalDevice, vmware.log.stringify(details)));
            } catch (error) {
                vmware.log("DEBUG", "plugin", "Error encontered while retrieving details for physical device {0}: {1}".format(physicalDevice, error));
            }

            return details;
        };

        var isPhysicalDeviceConnected = function(physicalDevice) {
            var details = getPhysicalDeviceDetails(physicalDevice);
            if (details) {
                return details.state == deviceControl.DeviceState.DS_CONNECTED;
            }
            return true;
        };

        var setEnabled = function(enabled) {
            // we only need to toggle the dialog for one type, but this is simpler
            if (enabled) {
                deviceButtonManager.setState("manage-cd", vmDeviceState(0));
                deviceButtonManager.setState("manage-floppy", vmDeviceState(1));
                deviceButtonManager.setState("manage-usb", vmDeviceState(2));
            } else {
                deviceButtonManager.setState("manage-cd", "defaultState");
                deviceButtonManager.setState("manage-floppy", "defaultState");
                deviceButtonManager.setState("manage-usb", "defaultState");
            }
        };

        var queue = (function() {
            var deferreds = {};

            var processCallback = function (virtualDevice, newState) {
                vmware.log("DEBUG", "devices", "Callback triggered for {0} with state {1}.".format(virtualDevice, newState));

                if (typeof(deferreds[virtualDevice]) === "undefined") {
                    deferreds[virtualDevice] = $.Deferred();
                }

                if (newState == deviceControl.DeviceState.DS_CONNECTED) {
                    deferreds[virtualDevice] = $.Deferred();
                } else {
                    deferreds[virtualDevice].resolve();
                }

                setEnabled(true);
            };

            var waitOnDisconnection = function (virtualDevice) {
                if (!isVirtualDeviceConnected(virtualDevice)) {
                    return $.when({});
                }

                if (typeof(deferreds[virtualDevice]) === "undefined") {
                    deferreds[virtualDevice] = $.Deferred();
                }

                return $.when(deferreds[virtualDevice]);
            };

            return {processCallback:processCallback, waitOnDisconnection:waitOnDisconnection};
        })();

        var disconnectDevice = function (virtualDevice) {
            setEnabled(false);

            try {
                vmware.log("TRACE", "plugin", "Disconnecting device {0}.".format(virtualDevice));

                deviceControl.disconnectDevice(virtualDevice);
            } catch (error) {
                vmware.log("DEBUG", "plugin", "Error encontered when disconnecting device {0}: {1}".format(virtualDevice, error));

                var errorMessage;
                if (error.message) {
                    errorMessage = error.message;
                } else {
                    errorMessage = "Couldn't disconnect device.";
                }
                consoleAlert(errorMessage.localize());
                return false;
            }

            return true;
        };

        var connectDevice = function(virtualDevice, backingType, clientPath) {
            setEnabled(false);

            // The actual connection process
            var doConnectDevice = function () {
                try {
                    vmware.log("TRACE", "plugin", "Connecting device {0}, {1}, {2}.".format(virtualDevice, clientPath, backingType));

                    deviceControl.connectDevice(virtualDevice, clientPath, backingType);
                } catch (error) {
                    vmware.log("DEBUG", "plugin", "Error encontered when connecting device {0}: {3}".format(virtualDevice, error));

                    var errorMessage;
                    if (error.message) {
                        errorMessage = error.message;
                    } else {
                        errorMessage = "Couldn't connect device.";
                    }
                    consoleAlert(errorMessage.localize());
                }
            };

            // Disconnect and wait if necessary
            if (isVirtualDeviceConnected(virtualDevice)) {
                if (disconnectDevice(virtualDevice)) {
                    queue.waitOnDisconnection(virtualDevice).then(doConnectDevice);
                }
            } else {
                doConnectDevice();
            }
        };

        return {getFileConnected: getFileConnected,
            isFileConnected: isFileConnected,
            getVirtualDevice: getVirtualDevice,
            isVirtualDeviceConnectedByMe: isVirtualDeviceConnectedByMe,
            hasClientBacking: hasClientBacking,
            getPhysicalDevices: getPhysicalDevices,
            getPhysicalDeviceDetails: getPhysicalDeviceDetails,
            isPhysicalDeviceConnected: isPhysicalDeviceConnected,
            isPhysicalDeviceConnectedByMe: isPhysicalDeviceConnectedByMe,
            connectDevice: connectDevice,
            disconnectDevice: disconnectDevice,
            processCallback: queue.processCallback,
            waitOnDisconnection: queue.waitOnDisconnection};
    })();

    var getRadioData = function(dialog) {
        var radioButtons = dialog.find('input[type="radio"]:checked');
        if (radioButtons.size() === 0) {
            return; // "Client Access Disabled" error case
        }

        var checkedItem = radioButtons[0];
        var newFile = dialog.find('input[type="text"]')[0];

        return {checkedItem: checkedItem.value, newFile: newFile.value};
    };

    var getCheckboxData = function(dialog) {
        var checkedItems = dialog.find('input[type="checkbox"]:checked');
        if (checkedItems.size() === 0) {
            return;
        }
        var valueList = new Array();
        for (var i = 0; i < checkedItems.size(); ++i) {
            valueList[i] = checkedItems[i].value;
        }
        return {checkedItems: valueList};
    };

    var updateDevice = function (deviceType, data) {
        if (!data) {
            return; // "Client Access Disabled" error case
        }
        var virtualDevice = devicePluginController.getVirtualDevice(deviceType);

        if (data.checkedItem == 'none') {
            if (devicePluginController.isVirtualDeviceConnectedByMe(virtualDevice)) { // x -> none
                devicePluginController.disconnectDevice(virtualDevice);
            } else { // none -> none
                return;
            }
        } else if (data.checkedItem == 'selectedFile') {
            if (!devicePluginController.isFileConnected(virtualDevice) // device x -> file y
                    || devicePluginController.getFileConnected(virtualDevice) != data.newFile) { // file x -> file y

                devicePluginController.connectDevice(virtualDevice, deviceControl.DeviceBacking.DB_FILE, data.newFile);
            } else { // file x -> file x
                return;
            }
        } else {
            var physicalDevice = data.checkedItem;
            if (!devicePluginController.isPhysicalDeviceConnectedByMe(physicalDevice)) { // x -> device y
                devicePluginController.connectDevice(virtualDevice, deviceControl.DeviceBacking.DB_PHYSICAL, physicalDevice);
            } else { // device x -> device x
                return;
            }
        }
    };

    var updateUsbDevice = function (data) {
        if (!data) {
            return;
        }
        for (var i = 0; i < data.checkedItems.length; ++i) {
            var physicalDevice = data.checkedItems[i];
            if (!devicePluginController.isPhysicalDeviceConnected(physicalDevice)) {
                devicePluginController.connectDevice(physicalDevice, deviceControl.DeviceBacking.DB_PHYSICAL, physicalDevice);
            } else if (devicePluginController.isPhysicalDeviceConnectedByMe(physicalDevice)) {
                devicePluginController.disconnectDevice(physicalDevice);
            }
        }
    };

    var createDeviceFormElement = function(type, parentElement, name, value, label, checked) {
        checked = typeof(checked) != 'undefined' ? checked : false;
        var input = $('<input type="' + type + '" name="' + name + '" value="' + value + '" ' + (checked ? 'checked="checked" ' : "") + '/>');
        parentElement.append($('<li></li>').append(input, label));
    };

    var buildDialogForm = function(deviceType, radioGroup) {
        var virtualDevice = devicePluginController.getVirtualDevice(deviceType);
        var content = $('<ul></ul>');

        var devices = devicePluginController.getPhysicalDevices(deviceType);
        if (devices) {
            createDeviceFormElement('radio', content, radioGroup, 'none', "None".localize(), !devicePluginController.isVirtualDeviceConnectedByMe(virtualDevice));
            for (var i = 0; i < devices.length; i++) {
                createDeviceFormElement('radio', content, radioGroup, devices[i], devices[i], devicePluginController.isPhysicalDeviceConnectedByMe(devices[i]));
            }

            var textInput = $('<div style="display: inline;"></div>')
                .append($('<input type="text" size="45" value="' + devicePluginController.getFileConnected(virtualDevice) + '" />'),
                        $('<br />'),
                        "Copy and paste the full path to the device you want to attach.".localize());
            createDeviceFormElement('radio', content, radioGroup, 'selectedFile', textInput, devicePluginController.isFileConnected(virtualDevice));
        } else {
            content.append($('<li></li>').text("Client device access is disabled.".localize()));
        }

        return content;
    };

    var manageCdDvd = function() {
        var cdDevice = devicePluginController.getVirtualDevice(deviceControl.DeviceType.DEVICE_CDROM);

        if (!cdDevice) {
            consoleAlert("No virtual CD/DVD device found".localize());
            return;
        }

        if (!devicePluginController.hasClientBacking(cdDevice)) {
            eic.setClientDeviceBacking(cdDevice, "iso");
        }

        hideConsole();

        vmware.dialog("Manage CDs/DVDs".localize(),
                $('<div></div>').append(
                        $('<p></p>').text("Choose the CD or DVD to attach to this VM. Only one device can be attached at a time.".localize()),
                        buildDialogForm(deviceControl.DeviceType.DEVICE_CDROM, 'cdGroup')),
                getRadioData,
                showConsole,
                updateDevice.curry(deviceControl.DeviceType.DEVICE_CDROM)
                );
        return true;
    };

    var manageFloppy = function() {
        var floppyDevice = devicePluginController.getVirtualDevice(deviceControl.DeviceType.DEVICE_FLOPPY);

        if (!floppyDevice) {
            consoleAlert("No virtual floppy device found".localize());
            return;
        }

        if (!devicePluginController.hasClientBacking(floppyDevice)) {
            eic.setClientDeviceBacking(floppyDevice, "floppy");
        }

        hideConsole();

        vmware.dialog("Manage Floppys".localize(),
                $('<div></div>').append(
                        $('<p></p>').text("Choose the Floppy drive to attach to this VM. Only one device can be attached at a time.".localize()),
                        buildDialogForm(deviceControl.DeviceType.DEVICE_FLOPPY, 'flpGroup')),
                getRadioData,
                showConsole,
                updateDevice.curry(deviceControl.DeviceType.DEVICE_FLOPPY)
                );
        return true;
    };

    var getUsbDeviceList = function() {
        var devices = devicePluginController.getPhysicalDevices(deviceControl.DeviceType.DEVICE_USB);
        var usbDeviceList = new Array();
        if (devices) {
            for (var i = 0; i < devices.length; i++) {
                if (!devicePluginController.isPhysicalDeviceConnected(devices[i])) {
                    usbDeviceList.push({connect:true, device:devices[i]});
                } else if (devicePluginController.isPhysicalDeviceConnectedByMe(devices[i])) {
                    usbDeviceList.push({connect:false, device:devices[i]});
                }
            }
            return usbDeviceList;
        } else {
            return null;
        }
    };

    var buildUsbDialogForm = function(usbDeviceList) {
        var content = $('<ul></ul>');
        for (var i = 0; i < usbDeviceList.length; i++) {
            var label = usbDeviceList[i].connect ?
                "Connect {0}".localize(devicePluginController.getPhysicalDeviceDetails(usbDeviceList[i].device).name) :
                "Disconnect {0}".localize(devicePluginController.getPhysicalDeviceDetails(usbDeviceList[i].device).name);
            createDeviceFormElement('checkbox', content, 'usbGroup', usbDeviceList[i].device, label, false);
        }
        return content;
    };

    var manageUsb = function() {
        var usbDeviceList = getUsbDeviceList();
        if (!usbDeviceList) {
            consoleAlert("Client device access is disabled.".localize());
            return
        }

        hideConsole();

        if (usbDeviceList.length == 0){
            vmware.alert("Manage USBs".localize(), "No client devices are available.".localize(), showConsole);
            return;
        }

        vmware.dialog("Manage USBs".localize(),
            $('<div></div>').append(
                    $('<p></p>').text("Choose the USB drive(s) to connect to or disconnect from this VM. One or more devices can be selected at a time.".localize()),
                    buildUsbDialogForm(usbDeviceList)),
            getCheckboxData,
            showConsole,
            updateUsbDevice);

        return true;
    };

    return {manageCdDvd:manageCdDvd, manageFloppy:manageFloppy, manageUsb:manageUsb, processCallback: devicePluginController.processCallback};
})();