/* Copyright 2009 VMware, Inc. All rights reserved. -- VMware Confidential */
    /**
     * This is the Javascript that will manage the VMRC plugin and send commands to it
     */
    var vmName  = null;
    var vmId    = null;
    var vmDevicesAvailable = null;
    var vmrc    = null;
    var deviceControl = null;
    var busyArea = null;
    var hideConsoleCallCount = 0;

    function initPlugin(config) {
        vmName = config.vmName;
        vmId = config.vmId;
        vmDevicesAvailable = config.vmDevicesAvailable;

        document.title = config.vmName ;

        setupVMRCObject();
        busyArea = document.getElementById("busyArea");
        initializei18n(config.buttonLabels, config.miscLabels, config.confirmationLabels);

        $("#vmName").text(vmName);

        setStatus("Connecting...".localize());

        span = $('#power-buttons');
        

        span = $('#plugin-buttons');
        pluginButtonManager = vmware.buttonManager({container: span})
            .createButton("ctrl-alt-del", {defaultState: {image: "./support/images/ctrl-alt-del-16x16-disabled.png", style: "cursor:auto", text: "Send Ctrl-Alt-Del (disabled)".localize()}, connected: {image: "./support/images/ctrl-alt-del-16x16.png", style: "cursor:pointer", text: "Send Ctrl-Alt-Del".localize()}})
            .registerHandler("ctrl-alt-del", function(event) {
                if (event.eventData.state == "connected") {
                    sendCtrlAltDelete();
                }
            })
            .createButton("fullscreen", {defaultState: {image: "./support/images/full-screen-16x16-disabled.png", style: "cursor:auto", text: "Full Screen (disabled)".localize()}, connected: {image: "./support/images/full-screen-16x16.png", style: "cursor:pointer", text: "Full Screen".localize()}})
            .registerHandler("fullscreen", function(event) {
                if (event.eventData.state == "connected") {
                    fullScreen();
                }
            })
            .createButton("fit-to-guest", {defaultState: {image: "./support/images/fit-to-guest-16x16-disabled.png", style: "cursor:auto", text: "Fit Window to Guest (disabled)".localize()}, connected: {image: "./support/images/fit-to-guest-16x16.png", style: "cursor:pointer", text: "Fit Window to Guest".localize()}})
            .registerHandler("fit-to-guest", function(event) {
                if (event.eventData.state == "connected") {
                    fitToGuest();
                }
            });

        var doInitPlugin = function () {
            startup();
            connect();
        };

        doInitPlugin.runWhen(isVmrcReady, 10);
    }

    function startup() {
        vmware.log("TRACE", "plugin", "Starting up MKS");
        vmrc.startup(vmrc.Mode.MKS, vmrc.MessageMode.EVENT_MESSAGES, 'usebrowserproxy=true;tunnelmks=true');

        attachConnectionStateChangeHandlers();
        attachDeviceConnectionStateChangeHandler();
    }

    function connect() {
        vmware.log("TRACE", "wrapper", "Acquiring ticket");

        
    }

    function connectControl(objectId, host, vmMoid, ticket) {
        vmware.log("TRACE", "plugin", "Connecting {0}".format(objectId));

        document.getElementById(objectId).connect(host, '', true, ticket, '', '', vmMoid, '', '');
    }

    function disconnect() {
        try {
            var status = vmrc.getConnectionState();

            if (status == vmrc.ConnectionState.CS_CONNECTED) {
                vmware.log("TRACE", "plugin", "Disconnecting MKS control");

                vmrc.disconnect();
            }
        } catch (error) {
            vmware.log("DEBUG", "plugin", "Error encontered when disconnecting MKS control: {0}".format(error));
        }

        try {
            var status = deviceControl.getConnectionState();

            if (status == deviceControl.ConnectionState.CS_CONNECTED) {
                vmware.log("TRACE", "plugin", "Disconnecting device control");

                deviceControl.disconnect();
            }
        } catch (error) {
            vmware.log("DEBUG", "plugin", "Error encontered when disconnecting device control: {0}".format(error));
        }
    }

    function uninit() {
        disconnect();

        vmrc.shutdown();
        deviceControl.shutdown();

        $(deviceControl).remove();
        $(vmrc).remove();
    }

    function createControl(id) {
        var c = $('<object></object>').attr('id', id);
        if ($.browser.msie) {
            c.attr('classid', 'CLSID:4AEA1010-0A0C-405E-9B74-767FC8A998CB');
        } else {
            c.attr('type', 'application/x-vmware-remote-console-2012');
        }

        c.addClass("console");
        c.addClass("hidden");

        return c;
    }

    //  classid attribute for IE, type attribute for Firefox
    function setupVMRCObject() {
        var vmrcObjectTag = document.getElementById("vmrcObject");

        createControl('vmrc').appendTo(vmrcObjectTag);
        createControl('deviceControl').appendTo(vmrcObjectTag);

        vmrc = vmware.normalizeConstants(document.getElementById('vmrc'));
        deviceControl = vmware.normalizeConstants(document.getElementById('deviceControl'));

        $(vmrc).removeClass("hidden");
    }

    function initializei18n(buttonLabels, miscLabels, confirmationLabels) {
        i18n.add("Send Ctrl-Alt-Del", buttonLabels[0]);
        i18n.add("Send Ctrl-Alt-Del (disabled)", buttonLabels[1]);
        i18n.add("Full Screen", buttonLabels[2]);
        i18n.add("Full Screen (disabled)", buttonLabels[3]);
        i18n.add("CD/DVD", buttonLabels[4]);
        i18n.add("CD/DVD (disabled)", buttonLabels[5]);
        i18n.add("Floppy", buttonLabels[6]);
        i18n.add("Floppy (disabled)", buttonLabels[7]);
        i18n.add("Fit Window to Guest", buttonLabels[8]);
        i18n.add("Fit Window to Guest (disabled)", buttonLabels[9]);
        i18n.add("USB", buttonLabels[10]);
        i18n.add("USB (disabled)", buttonLabels[11]);

        i18n.add("Connecting...", miscLabels[0]);
        i18n.add("Disconnected", miscLabels[1]);
        i18n.add("Failed to connect", miscLabels[2]);   // currently unused
        i18n.add("None", miscLabels[3]);
        i18n.add("Ok", miscLabels[4]);
        i18n.add("Cancel", miscLabels[5]);
        i18n.add("Browse", miscLabels[6]);              // currently unused
        i18n.add("Manage CDs/DVDs", miscLabels[7]);
        i18n.add("Choose the CD or DVD to attach to this VM. Only one device can be attached at a time.", miscLabels[8]);
        i18n.add("Manage Floppys", miscLabels[9]);
        i18n.add("Choose the Floppy drive to attach to this VM. Only one device can be attached at a time.", miscLabels[10]);
        i18n.add("Manage USBs", miscLabels[11]);
        i18n.add("Choose the USB drive(s) to connect to or disconnect from this VM. One or more devices can be selected at a time.", miscLabels[12]);
        i18n.add("Copy and paste the full path to the device you want to attach.", miscLabels[13]);
        i18n.add("Client device access is disabled.", miscLabels[14]);
        i18n.add("Device", miscLabels[15]);
        i18n.add("Error", miscLabels[16]);
        i18n.add("Operation failed. Please verify the vCD session is still active.", miscLabels[17]);
        i18n.add("No client devices are available.", miscLabels[18]);
        i18n.add("Connect {0}", miscLabels[19]);
        i18n.add("Disconnect {0}", miscLabels[20]);
        i18n.add("No virtual CD/DVD device found", miscLabels[21]);
        i18n.add("No virtual floppy device found", miscLabels[22]);

        i18n.add("Confirm Power On", confirmationLabels[0]);
        i18n.add("Power On Virtual Machine {0}?", confirmationLabels[1]);
        i18n.add("Confirm Suspend", confirmationLabels[2]);
        i18n.add("Suspend Virtual Machine {0}?", confirmationLabels[3]);
        i18n.add("Confirm Power Off", confirmationLabels[4]);
        i18n.add("Power Off Virtual Machine {0}?", confirmationLabels[5]);
        i18n.add("Confirm Reset", confirmationLabels[6]);
        i18n.add("Reset Virtual Machine {0}?", confirmationLabels[7]);
        i18n.add("Power On", confirmationLabels[8]);
        i18n.add("Suspend", confirmationLabels[9]);
        i18n.add("Power Off", confirmationLabels[10]);
        i18n.add("Reset", confirmationLabels[11]);
        i18n.add("Stop requested", confirmationLabels[12]);
        i18n.add("Suspend requested", confirmationLabels[13]);
    }

    function setStatus(message) {
        vmware.log("TRACE", "wrapper", "Displaying message: {0}".format(message));

        busyArea.innerHTML = message;
    }

    // used when we get an error trying to do a power operation, e.g., user has right to open the console but not for power ops.
    function consoleAlert(message) {
        vmware.log("TRACE", "wrapper", "Displaying alert: {0}".format(message));

        hideConsole();
        vmware.alert("Error".localize(), message, showConsole);
    }

    function sendCtrlAltDelete() {
        if (vmrc != null) {
            vmrc.sendCAD();
        }
    }

    function fullScreen() {
        if (vmrc != null) {
            vmrc.setFullscreen(true);
        }
    }

    function fitToGuest() {
        if (vmrc != null) {
            // desired width and height for vmrc object
            var guestWidth = vmrc.screenWidth();
            var guestHeight = vmrc.screenHeight();

            // width and height consumed by other elements
            var cssWidth = 0;
            var cssHeight = 40;

            // guess of width and height used by browser
            var guessedBrowserWidth = 16;
            var guessedBrowserHeight = 32;

            // values to pass to resize
            var computedWidth = guestWidth + cssWidth + guessedBrowserWidth;
            var computedHeight = guestHeight + cssHeight + guessedBrowserHeight;

            vmware.log("TRACE", "wrapper", "Attempting to resize to {0}x{1} for guest size {2}x{3}".format(computedWidth, computedHeight, guestWidth, guestHeight));
            window.resizeTo(computedWidth, computedHeight);

            // values resize resulted in
            var resultingWidth = $(vmrc).width();
            var resultingHeight = $(vmrc).height();

            // difference from desired values
            var widthDelta = resultingWidth - guestWidth;
            var heightDelta = resultingHeight - guestHeight;

            vmware.log("TRACE", "wrapper", "Result of resize was a guest size of {0}x{1}, a difference of {2}x{3}".format(resultingWidth, resultingHeight, widthDelta, heightDelta));

            // acceptable difference; it's okay to be a little too big
            var acceptablePositiveDelta = 16;
            var acceptableNegativeDelta = 0;

            // resize again if we're off by an unacceptable amount
            if ((widthDelta > acceptablePositiveDelta || widthDelta < acceptableNegativeDelta) ||
                (heightDelta > acceptablePositiveDelta || heightDelta < acceptableNegativeDelta)) {
                // adjust the computed sizes by the delta
                var newComputedWidth = computedWidth - widthDelta;
                var newComputedHeight = computedHeight - heightDelta;

                vmware.log("TRACE", "wrapper", "Using delta values to resize to {0}x{1}".format(newComputedWidth, newComputedHeight));
                window.resizeTo(newComputedWidth, newComputedHeight);
            }
        }
    }

    function isVmrcReady() {
        return vmrc != null && vmrc.isReadyToStart();
    }

    function hideConsole() {
        hideConsoleCallCount++;
        // We could make this conditional on the transition from 0 to 1
        // active hide call, but doing it unconditionally seems safer
        $(vmrc).addClass("hidden");
    }

    function showConsole() {
        hideConsoleCallCount--;
        if (hideConsoleCallCount == 0) {
            // Only show the console when there are no outstanding hide
            // requests
            $(vmrc).removeClass("hidden");
         }
    }

    //
    // EVENT REGISTRATION/HANDLING
    //
    function logEvent(plugin, event, args) {
        vmware.log("DEBUG", "plugin-event", "{0}.{1}: {2}".format(plugin, event, Array.prototype.slice.call(args).join(', ')));
    }

    function attachConnectionStateChangeHandlers() {
        if ($.browser.msie) {
            vmrc.attachEvent("onConnectionStateChange", handleVmrcControlConnectionStateChange);
            deviceControl.attachEvent("onConnectionStateChange", handleDeviceControlConnectionStateChange);
        } else {
            vmrc.onConnectionStateChange = handleVmrcControlConnectionStateChange;
            deviceControl.onConnectionStateChange = handleDeviceControlConnectionStateChange;
        }
    }

    function attachDeviceConnectionStateChangeHandler() {
        if ($.browser.msie) {
            deviceControl.attachEvent("onDeviceStateChange", handleClientDeviceStateChange);
        } else {
            deviceControl.onDeviceStateChange = handleClientDeviceStateChange;
        }
    }

    function handleVmrcControlConnectionStateChange(connectionState) {
        logEvent("vmrc", "onConnectionStateChange", arguments);

        if (connectionState == vmrc.ConnectionState.CS_CONNECTED) {
            setStatus("");

            pluginButtonManager.setState("fullscreen", "connected");
            pluginButtonManager.setState("ctrl-alt-del", "connected");
            pluginButtonManager.setState("fit-to-guest", "connected");

            deviceControl.startup(deviceControl.Mode.DEVICES, deviceControl.MessageMode.EVENT_MESSAGES, 'usebrowserproxy=true;tunnelmks=true');
            eic.acquireTicket(deviceControl.id);
        } else {
            setStatus("Disconnected".localize());
        }
    }

    function vmDeviceState(dev) {
        return vmDevicesAvailable[dev] ? "connected" : "defaultState";
    }

    function handleDeviceControlConnectionStateChange(connectionState) {
        logEvent("deviceControl", "onConnectionStateChange", arguments);

        if (connectionState == deviceControl.ConnectionState.CS_CONNECTED) {
            deviceButtonManager.setState("manage-cd", vmDeviceState(0));
            deviceButtonManager.setState("manage-floppy", vmDeviceState(1));
            deviceButtonManager.setState("manage-usb", vmDeviceState(2));
        }
    }

    function handleClientDeviceStateChange(deviceConnectionState, hostname, datacenter, vmID, virtualDeviceKey, physicalClientDeviceKey, userRequested, reason) {
        logEvent("deviceControl", "onDeviceStateChange", arguments);

        hideConsole();
        vmware.alert("Device".localize(), reason.localize(), showConsole);

        dmc.processCallback(virtualDeviceKey, deviceConnectionState);
    };

    //
    // END EVENT REGISTRATION/HANDLING
    //
