﻿#=================================================================================================================================================================================================
# NAME:           Get-ObjectContainerStats.ps1
# AUTHOR:         Dean Grant
# DATE:           26/04/2012
# VERSION:        1.2
#
# COMMENTS:       Retrieves statistical information for each virtual machine in a vCenter Server container object using the closest available statistical interval and exports to a CSV File.
# REVISION:       Removes any dependencies on virtual hardware version in collecting statistics for the inventory object, by collecting metrics individually for each guest VM.
#
# USAGE:          ./Get-ObjectContainerStats.ps1      
#=================================================================================================================================================================================================

# Powershell variables required to be completed to store vCenter Server and output information 
$VCenter = "king-vcenter-01.scc-zip.net" # Enter the vCenter Server hostname or IP address 
$ObjectName = "" #Enter the name of the vCenter Server container object 
$CSVLocation = "C:\Users\phil.spencer\Documents\" # Enter the output folder to export the CSV file.
$CSVFileName = "vm-" # Enter the filename for the exported CSV file. 

# Powershell variable to add the timestamp as a suffix for the filename.
$Date = (get-date).toString('ddMMyyyy_HHmm')  

# Adds registered vSphere PowerCLI snap-ins to the current session
Add-PSsnapin VMware.VimAutomation.Core >$null

# Connect to the vCenter Server
Connect-VIServer $VCenter >$null

# Collects all virtual machines in the specified vCenter Server container object 
$VM = Get-VM -Location ($ObjectName) # The vCenter Server container object may be modified to be a folder, resource pool, cluster or datacenter.

      ForEach ($i in $VM) 
            {
            # Retrieves the statistical infromation for each virtual machine for the previous 24 hours using a 5 minute interval and exports to a CSV file.
            $Metrics = Get-VM $i.Name | Get-StatType 
            $Stats = Get-Stat -Entity $i.Name -Stat $metrics  -Start (Date).AddDays(-1) -Finish (Date)
            $Stats | Select-Object Entity, Timestamp, MetricID, Value, Unit | Export-Csv ($CSVLocation + $CSVFileName + "_" + $Date + ".csv")  -NoTypeInformation
            } 
