function Get-FirewallDetails ($config)
{
    foreach ($fwrule in $config.firewall){
        $protocol = $fwrule.application.service.protocol
        $fwrule.SetAttribute("protocol",$protocol)
        $port = $fwrule.application.service.port
        $fwrule.SetAttribute("port",$port)
        $srcport = $fwrule.application.service.sourcePort
        $fwrule.SetAttribute("sourcePort",$srcport)
        $destex = $fwrule.destination.exclude
        $fwrule.SetAttribute("destExclude",$destex)
        $destip = $fwrule.destination.ipAddress
        $fwrule.SetAttribute("destIP",$destip)
        $srcex = $fwrule.source.exclude
        $fwrule.SetAttribute("srcExclude",$srcex)
        $srcip = $fwrule.source.ipAddress
        $fwrule.SetAttribute("srcIP",$srcip)
    }
}

function Get-NatDetails ($config)
{
    foreach ($natrule in $config.nat){
        $Originalip = $natrule.GatewayNatRule.OriginalIP
        $Translatedip = $natrule.GatewayNatRule.TranslatedIP
        $appliedon = $natrule.GatewayNatRule.interface.name
        $OriginalPort = $natrule.GatewayNatRule.originalport
        $TranslatedPort = $natrule.GatewayNatRule.TranslatedPort
        $Protocol = $natrule.GatewayNatRule.Protocol
        $natrule.SetAttribute("OriginalIP", $Originalip)
        $natrule.SetAttribute("TranslatedIP", $Translatedip)
        $natrule.SetAttribute("AppliedOn", $appliedon)
        $natrule.SetAttribute("OriginalPort", $OriginalPort)
        $natrule.SetAttribute("TranslatedPort", $TranslatedPort)
        $natrule.SetAttribute("Protocol", $Protocol)
    }
}

function Get-EdgeConfig ($EdgeGateway)
{
    $Edgeview = $EdgeGateway | get-ciview

    $webclient = New-Object system.net.webclient

    $webclient.Headers.Add("x-vcloud-authorization",$EdgeView.Client.SessionKey)

    $webclient.Headers.Add("accept","application/*+xml;version=32.0")

    $edgeview.id -match "(?<=urn:vcloud:gateway:).*"

    $edgeID = $Matches[0]
    
    $requrl = $global:DefaultCIServers.name + "/network/edges/" + $edgeID
    Write-Host $requrl
    [xml]$EGWConfXML = $webclient.DownloadString($requrl)

    $Holder = "" | Select Firewall,NAT,LoadBalancer,DHCP

    $Holder.Firewall =
    $EGWConfXML.edge.features.firewall.firewallrules.firewallrule

    $Holder.NAT =
    $EGWConfXML.edge.features.nat.natrules.natrule

    $Holder.LoadBalancer = 
    $EGWConfXML.edge.features.LoadBalancer

    $Holder.DHCP = 
    $EGWConfXML.edge.features.DHCP

    Return $Holder
}

